//==========================================================================
//	This source code is part of the Agent Simulation Test Bed system which
//	was developed for simulating interaction among agents who are attempting 
//	to acquire information from each other and learn about each other. 
//
//	The system was created at North Carolina State Univeristy, 
//	Department of Computer Science between September and December 2001. 
//	It was implemented by Michael J. Suralik based on the specifications 
//	developed by Pinar Yolum and Bin Yu, who were conducting research under 
//   the guidance of Dr. Munindar Singh. 
//==========================================================================

package edu.ncsu.simulators.agent;

import net.n3.nanoxml.*;

import java.io.StreamTokenizer;
import java.io.StringReader;
import java.util.Enumeration;
import java.util.Vector;


public final class Neighbors extends Vector
{	
	private static final char DELIMITER = ' ';
	
	//========================================================================			

	public Neighbors()
	{	
	}
	
	//========================================================================	

	public void addNeighbor( int id )
	{
		addElement( new Integer(id) );
	}
	
	//========================================================================	

	public boolean contains ( int id )
	{
		return contains( new Integer(id) );
	}

	//========================================================================	
	/**
	 * Returns a deep copy of this neighbors object
	 */
	public Neighbors copy()
	{
		Neighbors copy = new Neighbors();
		
		for ( int i=0; i<this.size(); i++ )
		{
			Integer copiedElement = new Integer( ((Integer)this.elementAt(i)).intValue() );
			copy.add( i, copiedElement );
		}
		
		return copy;		
	}
	
	//========================================================================	
	
	public static Neighbors parse( IXMLElement xmlNeighbors )
	{	
		Neighbors neighbors = new Neighbors();

		try
		{			
			String xmlNeighborsContent = xmlNeighbors.getContent();

			if ( xmlNeighborsContent != null )
			{
				StringReader reader = new StringReader( xmlNeighborsContent );
				StreamTokenizer tokenizer = new StreamTokenizer( reader );
				tokenizer.whitespaceChars( DELIMITER, DELIMITER );	

				while ( tokenizer.nextToken() != StreamTokenizer.TT_EOF )
		    		{									
					Integer number = new Integer( new Double(tokenizer.nval).intValue() );
					neighbors.add( number );
				}	
			}
		}
		catch ( Exception e )
		{
			e.printStackTrace();
		}		

		return neighbors;
	}
	
	//========================================================================
	
	public final void toXML( IXMLElement xmlNeighbors )
	{
		StringBuffer sbuf = new StringBuffer();
		Enumeration enum = this.elements();
		
		while( enum.hasMoreElements() )
		{
			sbuf.append( ((Integer)enum.nextElement()).toString() );
			
			if ( enum.hasMoreElements() )
			{
				sbuf.append( DELIMITER );
			}
		}
		
		xmlNeighbors.setContent( sbuf.toString() );
	}		
	
	//========================================================================		
}