
package edu.ncsu.simulators.agent;

/*
 * RatingAgentModel is a hierarchical model by which one agent models 
 * the other. This hierarchical model is a graph isomorphic to the Hierarchical-
 * -Classification of categories. Each node in this graph is represented by a 
 * LocalReputation object.
 */
 
import java.util.*;

import edu.ncsu.simulators.classification.*;
import edu.ncsu.simulators.evaluation.*;
import edu.ncsu.simulators.evaluation.results.*;

public class RatingAgentModel extends AgentModel {
	HierarchicalClassification classification;
	Hashtable localReputations;
	
	public RatingAgentModel(HierarchicalClassification c) {
		super();
		classification = c;
		localReputations = new Hashtable();
		initialize();
	}
	
	private void initialize(){
		Iterator it = classification.getIterator();
		
		while (it.hasNext()) {
			HierarchicalCategory c = (HierarchicalCategory)it.next();
			localReputations.put(c,new LocalReputation());
		}
		id = (int)Math.round(Math.random()*10000);
	}
	
	public void setRating(HierarchicalCategory c, Rating r){
		if (c.isLeafCategory()) {
			LocalReputation lr = (LocalReputation)localReputations.get(c);
			lr.setRating(r);
		}
	}
			
	public Rating getRating(HierarchicalCategory c){
		Rating r = (Rating)((LocalReputation)localReputations.get(c)).getRating();
		
		if (r == null){
			Iterator ite = c.getChildren();
			float sum = (float)0.0;
			int count = 0;
			
			while (ite.hasNext()){
				HierarchicalCategory cat = (HierarchicalCategory)ite.next();
				
				sum += ((FloatValue)(this.getRating(cat).getValue())).getValue();
				count++;
			}
			
			r = new Rating(new FloatValue(sum/count));
			((LocalReputation)localReputations.get(c)).setRating(r);
		}
		
		return r;
	}

}
