package edu.ncsu.simulators.agent;

import net.n3.nanoxml.*;
import java.util.*;
import edu.ncsu.simulators.*;
import edu.ncsu.simulators.dialogs.*;
import edu.ncsu.simulators.vectors.lattice.*;
import edu.ncsu.simulators.vectors.*;
import edu.ncsu.simulators.policies.*;
import edu.ncsu.simulators.util.Heuristics;
import edu.ncsu.simulators.Constants;
import edu.ncsu.simulators.util.*;
import edu.ncsu.simulators.io.LogWriter;

import java.io.File;
import java.io.FileWriter;
import java.io.StreamTokenizer;
import java.io.StringReader;

/**
 * ServiceAgent.java
 * An agent that has a lattice expertise. 
 *
 * Created: Mon May  6 19:03:51 2002
 *
 * @author <a href="mailto:pyolum@eos.ncsu.edu">Pinar Yolum</a>
 * @version 1.0
 */

public class ServiceAgent extends Agent {

    public ServiceAgent (int i){
	super(i);
    }

    public ServiceAgent () {

    }
    //public static final double PROMOTION_THRESHOLD = 0.15;
    public static final double PROMOTION_VALUE = 0.6;

    private int ownServiceProviders = 0;
    private int ownGoodProviders = 0;
    private int numberOfUsefulPromotedProviders = 0;
    private int numberOfPromotedProviders = 0;

    /**
     * Get the value of ownGoodProviders.
     * @return value of ownGoodProviders.
     */
    public int getOwnGoodProviders() {
	return ownGoodProviders;
    }
    
    /**
     * Set the value of ownGoodProviders.
     * @param v  Value to assign to ownGoodProviders.
     */
    public void setOwnGoodProviders(int  v) {
	this.ownGoodProviders = v;
    }
    /**    
     * Get the value of ownServiceProviders.
     * @return value of ownServiceProviders.
     */
    public int getOwnServiceProviders() {
	return ownServiceProviders;
    }
    
    /**
     * Set the value of ownServiceProviders.
     * @param v  Value to assign to ownServiceProviders.
     */
    public void setOwnServiceProviders(int  v) {
	this.ownServiceProviders = v;
    }
    
    /**
     * Picks a role randomly.
     *
     * @param numberOfRoles an <code>int</code> value: Total number of roles
     * @return a <code>double</code> value: Picks one from the total number
     */
    public double generateQueryUniformDistribution(int numberOfRoles ){

	double roleNumber = random.nextDouble();

	roleNumber= Math.floor( roleNumber * numberOfRoles);

	//If random number picks 1, then roleNumber can actually be off by 1
	if (roleNumber == numberOfRoles) roleNumber--;
	return roleNumber;
	
    }	

    //The way it is now, it is not really poisson but for now it will do.
    public double generateQueryPoissonDistribution(int numberOfRole){
	
	double roleNumber = random.nextDouble();
	if (roleNumber <= 0.045) 
	    roleNumber = 0; 
	else if ( roleNumber <= 0.136) 
	    roleNumber = 1; 
	else if ( roleNumber <= 0.226) 
	    roleNumber = 2; 
	else if ( roleNumber <= 0.406) 
	    roleNumber = 3; 
	else if ( roleNumber <= 0.586) 
	    roleNumber = 4; 
	else if ( roleNumber <= 0.766) 
	    roleNumber = 5; 
	else if ( roleNumber <= 0.856) //0.856) 
	    roleNumber = 6; 
	else if ( roleNumber <= 0.946) //0.946 
	    roleNumber = 7;
	else roleNumber = 8;
	
	return roleNumber;
    }


    public double generateQuery16(int numberOfRole){
	
	double roleNumber = random.nextDouble();
	if (roleNumber <= 0.023) 
	    roleNumber = 0; 
	else if ( roleNumber <= 0.069) 
	    roleNumber = 1; 
	else if ( roleNumber <= 0.115) 
	    roleNumber = 2; 
	else if ( roleNumber <= 0.184) 
	    roleNumber = 3; 
	else if ( roleNumber <= 0.253) 
	    roleNumber = 4; 
	else if ( roleNumber <= 0.312) 
	    roleNumber = 5; 
	else if ( roleNumber <= 0.404) 
	    roleNumber = 6; 
	else if ( roleNumber <= 0.496) 
	    roleNumber = 7;
	else if ( roleNumber <= 0.588) 
	    roleNumber = 8; 
	else if ( roleNumber <= 0.680) 
	    roleNumber = 9; 
	else if ( roleNumber <= 0.749) 
	    roleNumber = 10; 
	else if ( roleNumber <= 0.818) 
	    roleNumber = 11; 
	else if ( roleNumber <= 0.887) 
	    roleNumber = 12; 
	else if ( roleNumber <= 0.933) 
	    roleNumber = 13; 
	else if ( roleNumber <= 0.979) 
	    roleNumber = 14;
	else roleNumber = 15;
	
	return roleNumber;
    }




    /*    public double generate(int numberOfLevels) {

	int total = BuildLattice.totalServiceTypes(numberOfLevels);
	int level = 0;

	for ( int i=0; i<numberOfLevels; i++) {
	    
	} 
	
	}*/



    /**
     * First picks out a role and then makes a query structure out of that.
     * Then logs it into the agent log file.
     *
     */
    public void generateQuery(){

	double roleNumber = generateQuery16(expertise.getSize());//PoissonDistribution
	long currentQueryID = counterQueriesGenerated++;	

	currentQuery = new Query();		

	currentQuery.add( new Double(roleNumber) );
	System.out.println ("Agent " + getID() + " generating service query: " + roleNumber);
	promoteAgents((new Double(roleNumber).intValue()));


	log( "-----------------------------------------------------------------" );
	log( " QUERY #" + currentQueryID );
	log( "-----------------------------------------------------------------" );
	log( "generated query #" + currentQueryID + " : (" + currentQuery.toTruncatedString() + ")" );

    }
    
    /**
     * This <code>askQuestion<\code> is very similar to Agent.askQuestion.
     * But here first we check if the agent itself knows the answer for the 
     * query. 
     */

    public void askQuestion() {			
	//Generates a new query and stores in it 'currentQuery' member
	generateQuery();

	//ownServiceProviders = 0;
	//ownGoodProviders = 0;

	int roleNumber = ((Double)currentQuery.elementAt(0)).intValue();

	// Create a new referral chain specifically for this question.
	referralChain = new ReferralChainDAG( currentQuery, getID() );

	// Initialize for a new query
	setAgentsWhomHaveBeenSentCurrentQuery.clear();

	// Find own answer first. 
	Answer myAnswer = answeringPolicy.generateAnswer(null, currentQuery, getExpertise(), this.interests, this.id, getNeighbors());
	LatticeExpertise myExpertise = (LatticeExpertise)getExpertise();
	ProviderList pList = (ProviderList) myExpertise.getProviders().get(new Integer (roleNumber));
	Hashtable promotionTable = myExpertise.getPromotionTable();
	WeightMatrix weights = myExpertise.getWeights();
	
	Hashtable twoRoles = null;

	Integer previousRole = null;
	Integer newRole = null;

	//This is a BAD way of doing this.
	LearningPolicy_Lattice l = new LearningPolicy_Lattice();


	if (myAnswer != null) { //.equals(Constants.NULL_ANSWER)) {
	    int providerID = ((Double)myAnswer.elementAt(0)).intValue();
	    log (" Agent already knows of a provider " + providerID );
	    ownServiceProviders ++;
	    boolean promoted = myExpertise.isPromoted(providerID, roleNumber);
	    if (promoted) {
		numberOfPromotedProviders++;
	    } // end of if ()


	    double grade = l.evaluateAnswer (currentQuery, myAnswer);
	    if ( grade >  Constants.SIMILARITY_THRESHOLD_FOR_EVALUATING_ANSWER){
		ownGoodProviders++;
		if (promoted) {
		    numberOfUsefulPromotedProviders++;
		} // end of if ()
	    }	
		

	    myExpertise.updateProviders (roleNumber, providerID, grade);

    	    //Update the weightmatrix if the answerer has been promoted from another role.
	    Object temp = promotionTable.get (new Integer(providerID));
	    if (temp  != null) //the agent is in the promotion table
		{
		    twoRoles = (Hashtable)temp;

		    previousRole = (Integer)twoRoles.get(new Integer(roleNumber));
		    
		    if (previousRole != null) {
			log("     Updating weights: " + previousRole.intValue()+ "-" + roleNumber);
			updateWeights(weights, previousRole.intValue(), roleNumber, grade);
		    }
		}

	 

	} // end of if ()
	else {
	    
	    Iterator iteratorNeighborsToAsk = askingPolicy.getNeighborsToAsk( currentQuery.copy(), neighbors.copy(), agentModels.copy() );

	    long referralChainID = counterReferralChains++;		

	    while (iteratorNeighborsToAsk.hasNext()) {

		Integer neighborID = (Integer)iteratorNeighborsToAsk.next();		

		// add the neighbor that we just sent the question to as a referral
		//System.out.println("Asking neighbor " + neighborID.intValue());
		
		referralChain.addReferral( getID(), neighborID.intValue() );		

		// We synchronize on this block to make sure that the 
		// setAgentsWhomHaveBeenSentCurrentQuery structure accurately reflects
		// those agents whom have been sent the query.
		// Otherwise, another thread could possibly send a message to
		// the same neighbor if it was referred by some other agent.	

		synchronized (setAgentsWhomHaveBeenSentCurrentQuery) {

		    sendCurrentQuery( neighborID.intValue(), referralChainID  );
		}

	    }
	} // end of if ()else
	

    }

    /** 
     *  After asking each query to neighbors, decide if any 
     *  service provider in the lattice can do if. 
     *  If so, promote them to the new role.  
     *  Put the promoted agents into the promotion
     *  table so that the weights in the lattice can be 
     *  adjusted based on their service.
     *  @param roleId The chosen agents will be promoted to this roleId.
     */ 
    public void promoteAgents(int roleID){

	
	//Find all roleID that point to this roleID.
	
	LatticeExpertise l = (LatticeExpertise)expertise;    
	WeightMatrix weights = l.getWeights();
	//Hashtable promotionTable = l.getPromotionTable();
	//List of roles from which an agent will be chosen.
	Iterator possibleRoles = weights.pointTo(roleID).iterator();

	Integer currentRole = null;
	ProviderListNode currentProvider = null;
	double roleWeight = 0.0;

	while (possibleRoles.hasNext()){
	    //Take the role
	    currentRole = (Integer)possibleRoles.next();
	    //Find all the providers for that role
	    ProviderList p = (ProviderList)l.getProviders().get(currentRole);
	    if (p == null ) {
		p = new ProviderList();
		
	    } // end of if ()
	    
	    Iterator  pList = p.iterator();
	    roleWeight = weights.find(currentRole.intValue(), roleID);

	    //Check if any one of the providers can be promoted.
	    while (pList.hasNext()){
		currentProvider = (ProviderListNode)pList.next();
		//PROMOTE if the weight between the two roles are high AND 
		//the agent is not already doing the second role
		if (Heuristics.isPromotable(currentProvider, roleWeight, Constants.PROMOTION_THRESHOLD) 
		    && (!l.providesService(currentProvider.getProviderId(), roleID)))
		    { 
			l.updatePromotionTable(currentProvider.getProviderId(), currentRole, new Integer(roleID));
			l.updateProviders(roleID, currentProvider.getProviderId(), PROMOTION_VALUE);
			//updateModel(currentProvider.getProviderId(), roleID);
			log("Provider " + currentProvider.getProviderId() + " promoted from " + currentRole + " to " + roleID + " with rolewieght " + roleWeight);
		    }
			
	    }
	    //l.setPromotionTable(promotionTable);

	}
    }

    /* When a new agent is promoted, its modeled expertise is updated so that
     * it is given an initial expertise in the new role as well.
     */
    //      public void updateModel(int providerId, int roleId){			\
    //  	AgentModel model = agentModels.getModel(new Integer(providerId));	\
	//  	Expertise oldExpertise = model.getExpertise();				\
	//  	oldExpertise.set(roleId, new Double(PROMOTION_VALUE));			\
	//  	model.setExpertise(oldExpertise);					\
	//      }

	public String getXMLTag(){

	    return Constants.XML_TAG_SERVICE_AGENT;
	}
    

    /**
     * Update the weights based on the grade
     *
     * @param weights a <code>WeightMatrix</code> value
     * @param previousRole an <code>int</code> value
     * @param newRole an <code>int</code> value
     * @param grade a <code>double</code> value
     */
    private void updateWeights (WeightMatrix weights, int previousRole, int newRole, double grade){
	//Change the method to update
	double  currentWeight = weights.find(previousRole, newRole);
	if (grade > 0.5) {
	    currentWeight += Constants.ALPHA * currentWeight;
	    currentWeight = Math.min (currentWeight, 1);
	}
    
	else {
	    currentWeight -= Constants.ALPHA * currentWeight;
	    currentWeight = Math.max (currentWeight, 0);
	}
	weights.updateWeight(previousRole, newRole, currentWeight);

    }

    /**
     * Gets the value of numberOfUsefulPromotedProviders
     *
     * @return the value of numberOfUsefulPromotedProviders
     */
    public int getNumberOfUsefulPromotedProviders() {
	return this.numberOfUsefulPromotedProviders;
    }

    /**
     * Sets the value of numberOfUsefulPromotedProviders
     *
     * @param argNumberOfUsefulPromotedProviders Value to assign to this.numberOfUsefulPromotedProviders
     */
    public void setNumberOfUsefulPromotedProviders(int argNumberOfUsefulPromotedProviders){
	this.numberOfUsefulPromotedProviders = argNumberOfUsefulPromotedProviders;
    }

    /**
     * Gets the value of numberOfPromotedProviders
     *
     * @return the value of numberOfPromotedProviders
     */
    public int getNumberOfPromotedProviders() {
	return this.numberOfPromotedProviders;
    }

    /**
     * Sets the value of numberOfPromotedProviders
     *
     * @param argNumberOfPromotedProviders Value to assign to this.numberOfPromotedProviders
     */
    public void setNumberOfPromotedProviders(int argNumberOfPromotedProviders){
	this.numberOfPromotedProviders = argNumberOfPromotedProviders;
    }


}// ServiceAgent

