package edu.ncsu.simulators.analysis;

/*
 * Bipartite.java
 *
 * Created on February 21, 2002, 3:09 PM
 */

import java.io.*;
import java.util.*;
/**
 *
 * @author <a href="mailto:pyolum@eos.ncsu.edu">Pinar Yolum</a>
 * @version 
 */
public class Bipartite {
  /** All of the methods here are static.
   *  Will be used for finding communities.
   */
    public Bipartite() {
    }
  
    /**
     *  <code>outdegreeGreaterThen</code> Return true if node's
     *  outdegree is > j
     *
     * @param a an <code>Adjacency</code> value
     * @param node an <code>int</code> value
     * @param j an <code>int</code> value
     * @return a <code>boolean</code> value
     */
    public static boolean outdegreeGreaterThen(Adjacency a, int node, int j){
	return (Metrics.numberOfOutgoingNodes( a, node) >= j);
    }

    /**
     * <code>intersection</code> Returns the intersection of a list of
     * sets
     *
     * @param v a <code>HashSet</code> value
     * @return a <code>TreeSet</code> value 
     */

    public static TreeSet intersection(HashSet v) {
	Iterator i = v.iterator();
	TreeSet intersection = (TreeSet)i.next();
	while (i.hasNext())
	    intersection.retainAll((TreeSet)i.next());
	return intersection;
    }



    /**
     *  <code>createSetVector</code> method Gets the nodes hub points
     *  to.  For each pointer, finds who else is pointing to it (a
     *  treeset) Puts all the treesets into a vector
     *
     * @param a an <code>Adjacency</code> value
     * @param hub an <code>int</code> value
     * @return a <code>Vector</code> value */

    public static Vector createSetVector(Adjacency a, int hub){
	Iterator pointers = Metrics.findOutgoingNodes(a, hub).iterator();
	return createSetVector(a, pointers);

    }


    /**
     * Describe <code>createSetVector</code> method here.
     *
     * @param a an <code>Adjacency</code> value
     * @param pointers an <code>Iterator</code> value
     * @return a <code>Vector</code> value
     */
    
    public static Vector createSetVector(Adjacency a, Iterator pointers){
	Vector sets = new Vector();

	while (pointers.hasNext()){
	    int temp = ((Integer)pointers.next()).intValue();
	    Set t = Metrics.findIncomingNodes(a, temp);
	    if (t!=null)
		sets.add(t);
	    else sets.add(new HashSet());

	}
	return sets;

  
    }

    /**
     * Describe <code>createSetofSets</code> method here.
     *
     * @param a an <code>Adjacency</code> value
     * @param pointers an <code>Iterator</code> value
     * @return a <code>HashSet</code> value
     */
    public static HashSet createSetofSets(Adjacency a, Iterator pointers){
	HashSet sets = new HashSet();
      
	while (pointers.hasNext()){
	    int temp = ((Integer)pointers.next()).intValue();
	    //System.out.println(temp);
	    Set t = Metrics.findIncomingNodes(a, temp);
	    if (t!=null)
		sets.add(t);
	    else sets.add(new HashSet());
	    //System.out.println(sets);
	}
	return sets;
    }
    //CLEAN THIS!

    /**
     *  <code>findPerms</code> method find k permutations of v
     *
     * @param v a <code>Vector</code> value
     * @param k an <code>int</code> value
     * @return a <code>Vector</code> value
     */

    public static Vector findPerms(Vector v, int k){
	Vector perms = new Vector();
	Vector tempvect = new Vector();

	HashSet tree = new HashSet();
	HashSet tree1 = new HashSet();

	if (k<=1) {
	    //Basically return what you started with
	    for (Iterator i= v.iterator(); i.hasNext();){
		HashSet h = new HashSet();
		h.add(i.next());
		perms.add(new HashSet(h));
	    }
	}
	else{
	    for (int t=0; t <= (v.size()-k) ; t++){
		tree.add(v.elementAt(t));
		//Find smaller permutations
		tempvect = findPerms(new Vector(v.subList(t+1, v.size())), k-1);
		Iterator temp = tempvect.iterator();
		while (temp.hasNext()){
		    tree1 = (HashSet)tree.clone(); 
		    tree1.addAll((HashSet)temp.next());
		    perms.addElement(tree1.clone());
		}
		tree.clear();
	    }
	}
	return perms;
    }
   

    /**
     *  <code>findBipartiteCore</code> method finds a core of size i by j and returns
     *  the set of nodes for it finding through hubs, the left side set
     *
     * @param a an <code>Adjacency</code> value
     * @param i an <code>int</code> value
     * @param j an <code>int</code> value
     * @param hub an <code>int</code> value
     * @return a <code>HashSet</code> value 
     */

    public static HashSet findBipartiteCore(Adjacency a, int i, int j, int hub){
    
	HashSet core = null;
	HashSet cores = new HashSet();
	HashSet s = null;
    
	//Centers = hub's neighbors
	TreeSet centers = Metrics.findOutgoingNodes(a, hub);
	TreeSet fans = new TreeSet();
    
	//centers size should be >= other indepent set's size (j)
	if (centers.size() >= j){
	    //perms generates a list of j combination of centers
	    Vector set2vector = new Vector(centers);
	    Iterator perms = findPerms(set2vector, j).iterator();
	    //a combination will work if the same i people point to all of them
	    while (perms.hasNext()) {
		HashSet h  = (HashSet)perms.next(); //Set of authorities (centers)
		s= createSetofSets(a, h.iterator());
		fans = intersection(s); //hubs
		if (fans.size() >= i){
		    core = h;
		    core.addAll(fans);
		    cores.add(core.clone());
		    core.clear();
		}
	    }
	}
    
	return cores;
    }

    /* Expand as in Kumar et al.
     * Add all nodes that are pointed by the hubs and the nodes that point to the 
     * authorities. (only if a node points to at least two authorities)
     *
     * It won't work this way. Differentiate between fans and centers
     */

    public static HashSet expandBipartiteCore(Adjacency a, HashSet core){

	HashSet newNodes = new HashSet();
	int currentNode = -1;
	
	Iterator i = core.iterator();
	while (i.hasNext()) {
	    currentNode = ((Integer)i.next()).intValue();
	    newNodes.addAll(Metrics.findOutgoingNodes(a, currentNode));
	    
	} // end of while ()
	
	return newNodes;
    }


    /**
     *  <code>findAllBipartiteCores</code> method Go through the whole
     *  list of nodes: for each node either find a community or return
     *  null
     *
     * @param a an <code>Adjacency</code> value
     * @param i an <code>int</code> value
     * @param j an <code>int</code> value
     * @return a <code>HashSet</code> value 
     */
    public static HashSet findAllBipartiteCores(Adjacency a, int i, int j){
    
	HashSet cores = new HashSet();
	Set core = new HashSet();
	//This implementation is not removing nodes that already appear in a core
	//Should it?
	for (Enumeration e = a.keys(); e.hasMoreElements();){
	    core = findBipartiteCore(a, i, j, ((Integer)e.nextElement()).intValue());
	    if (core != null)
		cores.addAll(core);
	}  
	return cores;  
    }

    /**
     * Returns the list of producers
     *
     * @param a an <code>Adjacency</code> value
     * @return a <code>HashSet</code> value
     */
    public static HashSet findProducers(Adjacency a) {

	HashSet h = new HashSet();
	for (int i=0; i<a.size(); i++) {
	    if (Metrics.numberOfOutgoingNodes(a, i) == 0) {
		h.add(new Integer(i));
	    }
	} // end of for ()
	
	return h;
    }

    /**
     * Returns the number of edges that 
     *
     * @param a an <code>Adjacency</code> value
     * @return an <code>int</code> value
     */
    public static int offBipartite(Adjacency a) {

	HashSet h = findProducers(a);
	Integer n = null;

	int offBy = 0;
	
	//check all the nodes to see how many off them are off bipartite
	for (int i=0; i<a.size(); i++) {
	    n = new Integer(i);
	    if (!h.contains(n)) {
		//System.out.println(i + " difference " + a.difference(a.get(i), h));
		
		offBy += a.difference(a.get(i), h);
	    } // end of if ()
	    
	    
	} // end of for ()
	
	return offBy;
    }

}




