/*
 * DoubleVector.java
 *
 * Created on August 18, 2001, 12:36 PM
 */

package edu.ncsu.simulators.analysis;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Collections;

/**
 *
 * @author  pirnal
 * @version 
 */
public class DoubleVector  {
  /** This is a regular vector, except all elements are Doubles */

    private Vector v = new Vector();
    /** Creates new DoubleVector */
    public DoubleVector() {
    }
    
    public void set(int n, double x){
        v.set(n, new Double(x));
    }
    public void add(double x){
        v.add(new Double(x));
    }

    public void add(int i, double x){
        v.add(i, new Double(x));
    }
    public void add(Double x){
        v.add(x);
    }
    public double elementAt(int n){
        return ((Double)v.get(n)).doubleValue();
    }
    public int size(){
        return v.size();
    }
    public void clear(){
        v.clear();
    }

    public Enumeration elements(){

	return v.elements();
    }
    

    /** How similar are two vectors-- this is Euclidean distance
     **/
    public double howSimilartoVector(DoubleVector q){
        double sum=0.0;
        for (int i=0; i<size(); i++){
            sum+= Math.pow((this.elementAt(i)-q.elementAt(i)), 2);
            //System.out.println("distance is" + sum);
        }
        return 
            (Math.pow(Math.E, (-1*sum))-Math.pow(Math.E, (-1*size())))/
                (1-(Math.pow(Math.E, (-1*size()))));
    }
    

    /** How similar are two vectors-- this is actual cosine
     **/
    public double howSimilartoVector1(DoubleVector q){
        double sum = 0; //sum of individual items in the vectors
        double sumofthesecondvector=0;
        double sumofthefirstvector =0;
        for (int i=0; i<q.size();i++){
            sumofthefirstvector += Math.pow(q.elementAt(i),2.0);
            sumofthesecondvector += Math.pow(this.elementAt(i),2.0);
            sum += (this.elementAt(i)*q.elementAt(i));
        }
        return sum / Math.sqrt(sumofthefirstvector*sumofthesecondvector);
    }
    /** How similar are two vectors-- this is simiilar to cosine
     *but multiplies the denominator by n 
     **/
    
    public double howSimilartoExpertise(DoubleVector e){
        double sum = 0; //sum of individual items in the vectors
        double sumofthesecondvector=0;
        double sumofthefirstvector =0;

        //System.out.println("size of the second vector" + e.size());

        for (int i=0; i<this.size();i++){
            //sumofthefirstvector += Math.pow(e.elementAt(i),2.0);
            sumofthesecondvector += Math.pow(this.elementAt(i),2.0);
            sum += (this.elementAt(i)*e.elementAt(i));
            
        }

        return sum / Math.sqrt(e.size()*sumofthesecondvector);
    }
    
    /** This is the new interest to expertise similarity using averages
     *but punishing the negatives
     */
    public double howSimilartoExpertise2(DoubleVector e){
        double sum = 0; //sum of individual items in the vectors
        //double sumofthesecondvector=0;
        //double sumofthefirstvector =0;
        double temp = 0;
        for (int i=0; i<e.size();i++){
            //sumofthefirstvector += Math.pow(e.elementAt(i),2.0);
            //sumofthesecondvector += Math.pow(this.elementAt(i),2.0);
            temp = (e.elementAt(i) - this.elementAt(i));
            if (temp<0) temp=e.size()*temp;
            sum += temp;
            
        }
        //System.out.println("similarity s "+ (0.5 + (sum /e.size())) );
        return (0.5 + (sum /(2*e.size())));
    }
    
    /**tolerance is how much an element can be changed
     */
    public DoubleVector changeVector(double tolerance){
        DoubleVector d = new DoubleVector();
        for (int i=0;i<this.size();i++)
            d.set(i, 
            this.elementAt(i)-tolerance +
                (Math.random()*(2*tolerance)));
        return d;
    }
    public void estimatedUpdate(double factor, DoubleVector d){
        for (int i=0;i<=d.size();i++)
            set(i, (1-factor)*elementAt(i)+
                factor*d.elementAt(i));
    }
    public double mean(){
        double sum=0;
        for (int i=0;i<size();i++)
            sum +=elementAt(i);
        return sum/size();
    }   
    
    public double best(int i){
        Collections.sort(v);
        return elementAt(i);
    }    
    public double median(){
        Collections.sort(v);
        if ((v.size() % 2) == 1)
            return elementAt((v.size()-1)/2); //15se 8i
        else return (elementAt(v.size()/2) + elementAt((v.size()/2)-1))/2; //14se (7+8)/2
    }    
    public double standardDeviation(){
        double sum=0;
        double m=mean();
        for (int i=0;i<size();i++)
            sum += Math.pow((elementAt(i)-m), 2);
        return Math.sqrt(sum/(size()-1));
        
    }
    public double generality(){
        return mean()*(1-standardDeviation());
    }
    public void print(){
        for (int i=0; i<size(); i++)
            System.out.print(v.get(i) + " ");
    }

    /**
     * Compares the elements of this vector with the second
     * vector. Returns true if the difference is <= diff
     * for all elements.
     *
     * @param second a <code>DoubleVector</code> value
     * @param diff a <code>double</code> value
     * @return a <code>boolean</code> value
     */
    public boolean lessDifferent(DoubleVector second, double diff) {
	
	boolean more = false;
	int i = 0;
	while ((i<size()) && (!more)) {
	    more = (Math.abs(elementAt(i)-second.elementAt(i)) > diff) ;
	    i++;
	} // end of while ()
	
	return !more;

    }
    



    	
}
