// Created on October 20, 2001, 5:20 PM
//

package edu.ncsu.simulators.analysis;

import java.util.*;
import java.io.*;
import net.n3.nanoxml.*;
import edu.ncsu.simulators.Constants;
import edu.ncsu.simulators.vectors.*;
import edu.ncsu.simulators.agent.*;

public class GraphReader {
    /** Reads in a neighbor state file and generated adjacency,
     *  expertise table, etc.  
    */

    private Adjacency adjacency = new Adjacency();

    private VectorTable expertiseTable = new VectorTable();
    private VectorTable interestTable = new VectorTable();

   
    private Vector agents = new Vector();
    private int numDimensions= 0;
    private int numAgents;


    private IntHashTable models = new IntHashTable();

    /** Creates new Reader */
    public GraphReader() {
    }    

    /**
     * Get the value of agentModels.
     * @return value of agentModels.
     */
    public IntHashTable getModels() {
	return models;
    }
    
    /**
     * Set the value of agentModels.
     * @param v  Value to assign to agentModels.
     */
    public void setModels( IntHashTable  v) {
	this.models = v;
    }
    


    public Vector getAgents(){
        return agents;
    }
    public Adjacency getAdjacency(){
        return adjacency;
    }
    public VectorTable getInterestTable(){
        return interestTable;
    }
    
    public VectorTable getExpertiseTable(){
        return expertiseTable;
    }
    
    // This is mostly copied from Mike's code to parse the thing	
    public void parse( File inputFile, boolean models)
    {				
	try {

	    int i=0;
	    IXMLParser parser = XMLParserFactory.createDefaultXMLParser();
	    IXMLReader reader = new StdXMLReader(new FileInputStream(inputFile));
	    parser.setReader(reader);
	    IXMLElement xmlAgents = (IXMLElement) parser.parse();
			
	    Properties attributes = xmlAgents.getAttributes();
			
	    // Set the number of dimensions
	    this.numDimensions = Integer.parseInt( attributes.getProperty(Constants.XML_ATTRIBUTE_DIMENSIONS) );
			
	    int numAgents = Integer.parseInt( attributes.getProperty(Constants.XML_ATTRIBUTE_POPULATION_SIZE) );		
			
	    //agents = new Agent[numAgents];
	    int agentCounter = 0;
			
	    Enumeration enumChildrenOfAgents = xmlAgents.getChildren().elements();
			
	    while( enumChildrenOfAgents.hasMoreElements()) {

		IXMLElement xmlChildOfAgents = (IXMLElement)enumChildrenOfAgents.nextElement();
		//System.out.println(xmlChildOfAgents.getName());
		if ( xmlChildOfAgents.getName().equals(Constants.XML_TAG_AGENT) || xmlChildOfAgents.getName().equals(Constants.XML_TAG_SERVICE_AGENT)) 
		    parseAgent( xmlChildOfAgents,true );
		
		else if ( xmlChildOfAgents.getName().equals(Constants.XML_TAG_AGENT_MODEL))
		    parseModel( xmlChildOfAgents );					
	    }		
                       
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
		
    }	
	
    public  void parseAgent( IXMLElement xmlAgent, boolean setModels) {

	//Agent agent = null;
	int agentID = -1;				
	DoubleVector expertise = new DoubleVector();
                
	DoubleVector interests = new DoubleVector();
	TreeSet neighbors = new TreeSet();			
	
	AgentModels agentModels = new AgentModels();		
	AgentModel currentModel = new AgentModel();

	Double number = null;		
		
	try {

	    // Parse all the attributes
	    Properties attributes = xmlAgent.getAttributes();
			
	    agentID = Integer.parseInt( attributes.getProperty(Constants.XML_ATTRIBUTE_ID) );
			
	    // Parse all the child elements
	    Vector vecChildElements = xmlAgent.getChildren();
			
	    Enumeration enumChildElements = vecChildElements.elements();

	    while( enumChildElements.hasMoreElements()) {

		IXMLElement xmlChildElement = (IXMLElement)enumChildElements.nextElement();
		// parse Expertise
		if ( xmlChildElement.getName().equalsIgnoreCase(Constants.XML_TAG_EXPERTISE)) {

		    //if the propoertise is not null then it is a lattice expertise
		    //if so, ignore it
	    	    Properties expertiseAttributes = xmlChildElement.getAttributes();
		    //System.out.println(expertiseAttributes);
		    if ( expertiseAttributes.size() == 0) {
			//System.out.println("vector expertise");
			
			expertise = 
			    (DoubleVector)parseFromXML( xmlChildElement, new DoubleVector() );
			
		    } // end of if ()
		    
		}

		// parse Interests
		else if (xmlChildElement.getName().equalsIgnoreCase(Constants.XML_TAG_INTERESTS))
		    interests =  parseFromXML( xmlChildElement, new DoubleVector() );
			
		// parse Neighbors
		else if (xmlChildElement.getName().equalsIgnoreCase(Constants.XML_TAG_NEIGHBORS))
		    neighbors = parseNeighbors( xmlChildElement );
	
		// parse Agent Model
		else if (xmlChildElement.getName().equalsIgnoreCase(Constants.XML_TAG_AGENT_MODEL)) {
		    
		    currentModel = parseModel( xmlChildElement);
		    //Only keep the models of the neighbors
		    if ( neighbors.contains(new Integer(currentModel.getID()))){
			 agentModels.addModel(currentModel);	     
		    } // end of if ()

		}
		else if (xmlChildElement.getName().equals(Constants.XML_TAG_POLICIES)) {

		    Enumeration enumChildrenOfPolicies = xmlChildElement.getChildren().elements();	
		    while( enumChildrenOfPolicies.hasMoreElements() ) {
			IXMLElement xmlChildOfPolicies = (IXMLElement)enumChildrenOfPolicies.nextElement();
		    }
		}
			
	    }
			
	    agents.add(new Integer(agentID));
                        
	    adjacency.put(agentID, neighbors);//interests, expertise

	    if ( setModels ) {
		models.put(agentID, agentModels);
		interestTable.put(agentID, interests);
		expertiseTable.put(agentID, expertise);
	    } // end of if ()
	}
	catch( Exception e ) {
	    e.printStackTrace();
	}			
		
    }	


    public TreeSet parseNeighbors( IXMLElement xmlNeighbors )
    {	
	TreeSet neighbors = new TreeSet();
		
	try
	    {	
		String xmlNeighborsContent = xmlNeighbors.getContent(); 
		if ( xmlNeighborsContent != null )
		    {
			StringReader reader = new StringReader( xmlNeighborsContent );
			StreamTokenizer tokenizer = new StreamTokenizer( reader );
			tokenizer.whitespaceChars( ' ', ' ' );	
			
			while ( tokenizer.nextToken() != StreamTokenizer.TT_EOF )
			    {									
				Integer number = new Integer( new Double(tokenizer.nval).intValue() );
				neighbors.add( number );
			    }
		    }
	    }
	catch ( Exception e )
	    {
		e.printStackTrace();
	    }
		
	return neighbors;
    }
        
    /**
	 * Parses the agent represented by the specified xml element.
	 */
    public AgentModel parseModel( IXMLElement xmlAgentModel )
    {
	AgentModel model = new AgentModel();
		
	int agentID = -1;				
	Expertise expertise = new DomainExpertise();
	//Interests interests = null;
	Interests interests = (Interests) new DomainInterests();		

	Sociability sociability = null;//new DomainSociability();
	double temp;
	Double number = null;		
		
	try
	    {	
		// Parse all the attributes
		Properties attributes = xmlAgentModel.getAttributes();
			
		agentID = Integer.parseInt( attributes.getProperty(Constants.XML_ATTRIBUTE_ID) );
		temp = Double.parseDouble( attributes.getProperty(Constants.XML_ATTRIBUTE_SOCIABILITY, Constants.XML_ATTRIBUTE_DEFAULT) );
		if (temp!=-1) { //scalar sociability
		    sociability = new ScalarSociability(temp);
		    
		} // end of if ()
		else {
		    sociability = new DomainSociability();
		} // end of else
		

	
		// Parse all the child elements
		Vector vecChildElements = xmlAgentModel.getChildren();
		Enumeration enumChildElements = vecChildElements.elements();
			
		while( enumChildElements.hasMoreElements() )
		    {
			IXMLElement xmlChildElement = (IXMLElement)enumChildElements.nextElement();
			
				// parse Expertise
			if ( xmlChildElement.getName().equalsIgnoreCase(Constants.XML_TAG_EXPERTISE) )
			    {				
				expertise = expertise.parse( xmlChildElement );

			    }
					
				// parse Interests
			else if ( xmlChildElement.getName().equalsIgnoreCase(Constants.XML_TAG_INTERESTS) )
			    {
				// interests = Interests.parse( xmlChildElement );
				interests = interests.parse( xmlChildElement ); // changed from the previous line
				// to the current line by Raghuram.
			    }

			else if ( xmlChildElement.getName().equalsIgnoreCase(Constants.XML_TAG_SOCIABILITY) )
			    {
				// interests = Interests.parse( xmlChildElement );
				sociability = sociability.parse( xmlChildElement ); 

			    }
		    }
		model.setID( agentID );
		model.setSociability( sociability );
		model.setExpertise( expertise );
		model.setInterests( interests );
			

	    }
	catch ( Exception e )
	    {
		e.printStackTrace();
	    }

	return model;
		
    }

    /**
     * Loads the file s.
     *
     * @param s a <code>String</code> value
     */
    public void load(String s, boolean models)
    {				
		
	File fileInput = null;
				
	try {

	    fileInput = new File(s);
	    parse( fileInput , models);			
	}
	catch( Exception e ) {

	    System.err.println( "Simulation File '" + fileInput.toString() + "' not found" );
	    System.exit(1);
	}	
    }

    public void load (String s) {
	load (s, false);
    }
    /**
     * Parse in a doublevector
     *
     * @param xmlVector an <code>IXMLElement</code> value
     * @param simulationVector a <code>DoubleVector</code> value
     * @return a <code>DoubleVector</code> value
     */
    public static DoubleVector parseFromXML( IXMLElement xmlVector, DoubleVector simulationVector )
    {	
	StringReader reader = new StringReader( xmlVector.getContent() );
	StreamTokenizer tokenizer = new StreamTokenizer( reader );
	tokenizer.whitespaceChars( ' ', ' ' );			

	try {			
	    while ( tokenizer.nextToken() != StreamTokenizer.TT_EOF ) {

		Double number = new Double(tokenizer.nval);
		simulationVector.add( number );
	    }	
	}
	catch ( Exception e ) {
	    e.printStackTrace();
	}
		
	return simulationVector;
    }	

}
