/*
 * RunAnalysis.java
 *
 * Created on October 20, 2001, 5:49 PM
 */

package edu.ncsu.simulators.analysis;


import java.util.*;
import java.io.*;
/**
 *
 * @author  pirnal
 * @version 
 */
public class RunAnalysis {
  /** This is the main class that calls quality and powerlaw things
   *  Should be modified to say just do these measurements
   *
   */

    /** Creates new RunAnalysis */
    public RunAnalysis() {
    }

    public static void main (String args[]) {
        GraphReader r = new GraphReader();
        
        r.load(args[0]);
                
        Adjacency adjacencyTable = r.getAdjacency();
        VectorTable interestTable =r.getInterestTable(); 
        VectorTable expertiseTable=r.getExpertiseTable();
	
        Vector agents = r.getAgents();

        Vector edges = adjacencyTable.getEdges();
        FileWriter outputFileWriter;
        Vector components = new Vector();
        double quality = 0.0;
        double interestSimilarity = 0.0; 
        double expertiseSimilarity = 0.0; 
        double nodequality = 0.0;
	Vector accuracy = new Vector();
        int numberofAgents = adjacencyTable.size();
	
	int[][] shortestPaths = new int[numberofAgents][numberofAgents];
        int[] d = new int[numberofAgents*2]; 
        int[] p = new int[numberofAgents*2];
	double avgPath = 0;
        
        for (int j=0; j<numberofAgents; j++){
	    ShortestPath.belmannford(adjacencyTable, 
			((Integer)agents.get(j)).intValue(), d, p);
            for (int k=0; k<numberofAgents; k++)
	      shortestPaths[j][k] = d[k];
          
	    }
	avgPath = ShortestPath.averageLength(shortestPaths, numberofAgents);
	//Find quality
	quality =  Metrics.qualityMedian(shortestPaths, agents, interestTable, expertiseTable, adjacencyTable);
	//Find direct quality
	nodequality =  Metrics.nodeQuality(edges, agents, interestTable, expertiseTable);
	//Find interest similarity clustering
	interestSimilarity = Metrics.interestSimilarity(interestTable, adjacencyTable, agents);
	//Find expertise similarity clustering
	expertiseSimilarity= Metrics.interestSimilarity(expertiseTable, adjacencyTable, agents);


       // Write everyting to their files in the log directory
       try 
	{
		outputFileWriter = new FileWriter( "interest-vs-quality.txt", true);
		outputFileWriter.write(new Double(interestSimilarity).toString());
                outputFileWriter.write("\t");
                outputFileWriter.write(new Double(quality).toString());
                outputFileWriter.write("\n");
                outputFileWriter.close();


		outputFileWriter = new FileWriter( "average shortest path.txt", true);
		outputFileWriter.write(new Double(avgPath).toString()+"\n");
                outputFileWriter.close();

		outputFileWriter = new FileWriter( "expertise-vs-quality.txt", true);
		outputFileWriter.write(new Double(expertiseSimilarity).toString());
                outputFileWriter.write("\t");
                outputFileWriter.write(new Double(quality).toString());
                outputFileWriter.write("\n");
                outputFileWriter.close();
	
                outputFileWriter = new FileWriter( "nodequality-vs-quality.txt", true);
                outputFileWriter.write(new Double(nodequality).toString());
		
                outputFileWriter.write("\t");
		outputFileWriter.write(new Double(quality).toString());
                outputFileWriter.write("\n");
                outputFileWriter.close();
                
                outputFileWriter = new FileWriter( "interest-vs-expertise.txt", true);
		outputFileWriter.write(new Double(interestSimilarity).toString());
                outputFileWriter.write("\t");
                outputFileWriter.write(new Double(expertiseSimilarity).toString());
                outputFileWriter.write("\n");
                outputFileWriter.close();

		
	} 

	catch( Exception e )
	{
		e.printStackTrace();
	}
    }

}
