package edu.ncsu.simulators.evaluation.results;

import edu.ncsu.simulators.evaluation.*;
import edu.ncsu.simulators.service.ServiceProvider;
import edu.ncsu.simulators.vectors.SimulationVector;

import java.util.*;

/* This Class represents one row of the "ScoreTable", which is a table of
 * agents vs the service providers they have scored.
 */
 
public class AgentScorings extends SimulationVector{

	int agentID;
	Hashtable association;
	
	public AgentScorings(int id){
		agentID = id;
		association = new Hashtable();
	}
	
	public AgentScorings(int id, Vector scores){
		this(id);
		addAll(scores);
	}
	
	public void addScore(ServiceProvider sp, Value val){
		add(new Score(this.agentID, sp, val));
	}
	
	public void addScore(Score score){
		if (score.getAgentID() != agentID) {
			System.out.println("score " + score + " is not given by agent " +
							   agentID + "!!");
		} else {
			add(score);
		}
	}
	
	public boolean add(Object score){
		if ( !(score instanceof Score) ){
			throw new ClassCastException("Attempted to add non-score object " +
										 "to AgentScorings!!");
		}
		
		add(score);
		association.put(((Score)score).getServiceProvider(),score);
		return true;
	}
	
	public Score getScoreForServiceProvider(ServiceProvider sp){
		
		return (Score)association.get(sp);
	}
		
		
	public Enumeration getRatedServiceProviders(){
//		Score score;
//		HashSet sps = new HashSet();
//		
//		for (Iterator ite = this.iterator(); ite.hasNext(); ){
//			score = (Score)ite.next();
//			sps.add(score.getServiceProvider());
//		}
//		
//		return sps.iterator();
//		
		return association.keys();
	}
	
	public Score getScoreForSP(ServiceProvider sp){
		return (Score)association.get(sp);
	}
	
	/*
	 * Compare two AgentScorings and return the distance between
	 * them
	 */
	public float difference(AgentScorings vec2){
		float val = (float)0.0;
		
		for (int i = 0; i < size(); i++){
			Score myScore = (Score)elementAt(i);
			ServiceProvider sp = (ServiceProvider)myScore.getServiceProvider();
			Score hisScore = vec2.getScoreForServiceProvider(sp);
			
			if ( hisScore != null ){
				val = ((FloatValue)myScore.difference(hisScore)).getValue();
				val *= val;
			}
		}
		
		
		val = (float)Math.sqrt(val);
		return val;
	}
		
		
}
