//==========================================================================
//	This source code is part of the Agent Simulation Test Bed system which
//	was developed for simulating interaction among agents who are attempting 
//	to acquire information from each other and learn about each other. 
//
//	The system was created at North Carolina State Univeristy, 
//	Department of Computer Science between September and December 2001. 
//	It was implemented by Michael J. Suralik based on the specifications 
//	developed by Pinar Yolum and Bin Yu, who were conducting research under 
//   the guidance of Dr. Munindar Singh. 
//==========================================================================

package edu.ncsu.simulators.io;

import edu.ncsu.simulators.Constants;

import java.io.*;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Vector;

/**
 * The LogWriter writes strings to a file in a nicely formatted fashion, with
 * the date and time stamp that is useful for logs.
 */
public class LogWriter
{	
    // The class maintains a collection of pointers to all the log file writers
    // so that we can flush and close all files with a single method call
    // when an exception occurs
    static private Vector vecWriters = new Vector();
	
	// We use a buffered writer to increase efficiency
    private BufferedWriter writer = null;

    //========================================================================				
	
    public LogWriter(FileWriter fileWriter) throws java.io.IOException
    {
	// I (MJS) tried 1024, 4096, 8192 for buffer sizes.
	// 4096 was the most efficient, 1024 was worse, and 8192 was the worst
	writer = new BufferedWriter( fileWriter, 4096 );
	vecWriters.addElement( fileWriter );
    }
	
    //========================================================================				
	
    public LogWriter(File file) throws java.io.IOException
    {
	this( new FileWriter(file) );
    }
	
    //========================================================================				
	
    public LogWriter(String filename) throws java.io.IOException
    {
	this( new FileWriter(filename) );
    }
	
    //========================================================================				
	
    public void log( String text )
    {
	try 
	    {
		String screenText = text;
		String logText = "[" + Calendar.getInstance().getTime() + "]  " + text + '\r' + '\n';
		
		if ( writer != null )
		    {
			writer.write( logText, 0, logText.length() );
		    }
		else
		    {
			System.out.println( "Attempted to write to a FileWriter that has been closed" );
		    }

	    }
	catch( Exception e )
	    {
		e.printStackTrace();
	    }		
    }

    //========================================================================		
    /**
     * Flush and close all logFileWriters
     */
    static public void closeAllFiles()
    {
	Enumeration enumLogWriters = vecWriters.elements();
		
	try
	    {
		while ( enumLogWriters.hasMoreElements() )
		    {
			LogWriter logWriter = (LogWriter)enumLogWriters.nextElement();
			logWriter.close();
		    }
	    }
	catch ( Exception e )
	    {
		System.out.println( "Error occured during LogWriter.closeAllFiles" );
		e.printStackTrace();
	    }
    }
	
    //========================================================================		
	
    public void close()
    {
	try
	    {
		writer.flush();			
		writer.close();
		writer = null;
	    }
	catch ( Exception e )
	    {
		System.out.println( "Error occured during LogFileWriter.close" );			
		e.printStackTrace();
	    }
    }
	
    //========================================================================			
}
