//==========================================================================
//	This source code is part of the Agent Simulation Test Bed system which
//	was developed for simulating interaction among agents who are attempting 
//	to acquire information from each other and learn about each other. 
//
//	The system was created at North Carolina State Univeristy, 
//	Department of Computer Science between September and December 2001. 
//	It was implemented by Michael J. Suralik based on the specifications 
//	developed by Pinar Yolum and Bin Yu, who were conducting research under 
//   the guidance of Dr. Munindar Singh. 
//==========================================================================

package edu.ncsu.simulators.io;

import edu.ncsu.simulators.agent.Agent;
import edu.ncsu.simulators.agent.AgentModel;

/**
 * The SimulationGenerator is the common interface that is used by all objects
 * that load and/or create the agents (and other objects) that are to be used 
 * by the Simulator.
 *
 */
public interface SimulationGenerator
{	
	//========================================================================			

	/**
	 * Returns the array of agents that inhabit this simulation's universe
	 */
	public Agent[] getAgents();
	
	/**
	 * Returns the number of dimensions in this simulation's universe of 
	 * discourse.  This will be the size of the Query, Interest, Expertise, 
	 * and Answer vectors.
	 */
	public int getDimensions();
	
	/**
	 * Returns the default agent model that an agent should use when modeling 
	 * another agent that it has encountered for the first time.
	 */
	public AgentModel getDefaultAgentModel();
	
	//========================================================================		
}