//==========================================================================
//	This source code is part of the Agent Simulation Test Bed system which
//	was developed for simulating interaction among agents who are attempting 
//	to acquire information from each other and learn about each other. 
//
//	The system was created at North Carolina State Univeristy, 
//	Department of Computer Science between September and December 2001. 
//	It was implemented by Michael J. Suralik based on the specifications 
//	developed by Pinar Yolum and Bin Yu, who were conducting research under 
//   the guidance of Dr. Munindar Singh. 
//==========================================================================
package edu.ncsu.simulators.policies;

import edu.ncsu.simulators.agent.AgentModel;
import edu.ncsu.simulators.agent.AgentModels;
import edu.ncsu.simulators.vectors.Answer;
import edu.ncsu.simulators.vectors.Query;
import edu.ncsu.simulators.vectors.Interests;
import edu.ncsu.simulators.vectors.Expertise;
import edu.ncsu.simulators.agent.Neighbors;

/**
 * Defines the interface between an agent and his policy for making decisions
 * regarding the answering of questions that have been posed to him.
 */
public interface AnsweringPolicy
{	
	/**
	 * Returns the answer generated by the answering agent in response to the 
	 * specified query.
	 *
	 * All 3 parameters are copies of the actual objects that they represent.  
	 * They may be altered in order to facilitate generating the return value.  
	 * However, altering the parameters will have no effect outside this method.
	 *
	 * @param modelQuestioner the model of the agent who made the query
	 * @param query the Query being made
	 * @param expertise the Expertise of the agent who is answering the query
	 * @return the answer generated by the answering agent in response to the 
	 * query; the answer should be NULL if the answering agent decides that it
	 * does not want to answer the questioning agent's query for any reason.
	 */
	public Answer generateAnswer( AgentModel modelQuestioner, Query query, Expertise expertise, Interests interest, int myId, Neighbors neighbors);
}
