package edu.ncsu.simulators.policies;

import java.util.*;
import edu.ncsu.simulators.vectors.lattice.*;
import edu.ncsu.simulators.vectors.*;
import edu.ncsu.simulators.agent.*;
import edu.ncsu.simulators.Constants;
import edu.ncsu.simulators.agent.Neighbors;

/**
 * AnsweringPolicy_Lattice.java
 * Checks the lattice and finds a provider from the lattice nodes.
 *
 * Created: Wed May  8 13:14:15 2002
 *
 * @author <a href="mailto:pyolum@eos.ncsu.edu">Pinar Yolum</a>
 * @version 1
 */

public class AnsweringPolicy_Lattice implements AnsweringPolicy{

    public AnsweringPolicy_Lattice (){
	
    }

    public Answer generateAnswer( AgentModel modelQuestioner, Query query, Expertise expertise, Interests interest, int myId, Neighbors neighbors){
	Answer newAnswer = null;

	double providerId = -1; //this is the provider to be recommended
	double bestAverage = 0.0; //this is the provider's average
	ProviderListNode currentNode = null;

	LatticeExpertise l = (LatticeExpertise)expertise;

	//roleNumber keeps the identifier for the role 
	int roleNumber = ((Double)(query.elementAt(0))).intValue();
	//pList has all the providers that have previously acted for that role
	//and their ratings.
	
	ProviderList pList = (ProviderList)l.getProviders().get(new Integer(roleNumber));

	if (pList != null) {
	    
	    Iterator i = pList.iterator();

	    //Look through all the providers and return the one that has the best average.
	    //This can later be eloborated by weighted average, number of interactions, etc.
	    while (i.hasNext()){
		currentNode = (ProviderListNode)i.next();
		if (bestAverage < currentNode.avgRating()) {
		    providerId = currentNode.getProviderId();
		    bestAverage = currentNode.avgRating();
		} //end while
	    }
	    //The bestAverage should still be better than some threshold
	    if ( bestAverage > Constants.SIMILARITY_THRESHOLD_FOR_ANSWERING_QUESTIONER) {
	    
		newAnswer = new Answer();
		newAnswer.add( new Double(providerId) ); 
	    } // end of if ()
	    

	    //System.out.println("Found a provider for role " + roleNumber + ": " +  providerId);

	} //end if	
	
	return newAnswer;
    }
	
}// AnsweringPolicy_Lattice
