//==========================================================================
//	This source code is part of the Agent Simulation Test Bed system which
//	was developed for simulating interaction among agents who are attempting 
//	to acquire information from each other and learn about each other. 
//
//	The system was created at North Carolina State Univeristy, 
//	Department of Computer Science between September and December 2001. 
//	It was implemented by Michael J. Suralik based on the specifications 
//	developed by Pinar Yolum and Bin Yu, who were conducting research under 
//   the guidance of Dr. Munindar Singh. 
//==========================================================================

package edu.ncsu.simulators.policies;

import edu.ncsu.simulators.agent.AgentModels;
import edu.ncsu.simulators.agent.Neighbors;
import edu.ncsu.simulators.vectors.Query;

import java.util.Iterator;

/**
 * Defines the interface between an agent and his policy for making decisions
 * regarding which questions he will ask which agents.
 */
public interface AskingPolicy
{		
	/**
	 * Return a collection of neighbors to whom the questioning agent should
	 * send the query (this MUST be a subset of the agents specified in the
	 * neighbors parameter).
	 *
	 * All 3 parameters are copies of the actual objects that they represent.  
	 * They may be altered in order to facilitate generating the return value.  
	 * However, altering the parameters will have no effect outside this method.
	 * 
	 * @param query the query that the questioning agent wants to send
	 * @param neighbors all of the neighbors whom the questioning agent can consider sending the query to
	 * @param agentModels all of the agent models that the questioning agent maintains
	 * @return a collection of neighbors whom the questioning agent should send
	 * the query to (this will be a subset of the agents specified in the neighbors parameter)
	 */ 
	public Iterator getNeighborsToAsk( Query query, Neighbors neighbors, AgentModels agentModels );
}