//==========================================================================
//	This source code is part of the Agent Simulation Test Bed system which
//	was developed for simulating interaction among agents who are attempting 
//	to acquire information from each other and learn about each other. 
//
//	The system was created at North Carolina State Univeristy, 
//	Department of Computer Science between September and December 2001. 
//	It was implemented by Michael J. Suralik based on the specifications 
//	developed by Pinar Yolum and Bin Yu, who were conducting research under 
//   the guidance of Dr. Munindar Singh. 
//==========================================================================
package edu.ncsu.simulators.policies;

import edu.ncsu.simulators.agent.AgentModels;
import edu.ncsu.simulators.agent.Neighbors;
import edu.ncsu.simulators.io.LogWriter;
import edu.ncsu.simulators.vectors.Expertise;
import edu.ncsu.simulators.vectors.Interests;

/**
 * Defines the interface between an agent and his policy regarding dropping 
 * existing neighbors and/or adding new neighbors.
 */
public interface ChangingNeighborsPolicy
{	
	/**
	 * Modifies the neighbors of an agent by adding and/or removing agents.
	 *
	 * The other 3 parameters are copies of the actual objects that they represent.  
	 * They may be altered in order to facilitate generating the return value.  
	 * However, altering the parameters will have no effect outside this method.
	 *
	 * @param neighbors the neighbors of the agent who is changing his set of neighbors.
	 * @param agentModels the models of all the agents that the agent who is making the change is aware of
	 * @param expertise the expertise of agent who is making the change
	 * @param interests the expertise of the agent who is making the change.
	 */
	public void changeNeighbors( Neighbors neighbors, AgentModels agentModels, Expertise expertise, Interests interests );
	
	/**
	 * Sets the FileWriter that the policy can use to output the results
	 */
	public void setLogWriter( LogWriter logWriter );	
}