package edu.ncsu.simulators.policies;

import edu.ncsu.simulators.Constants;
import edu.ncsu.simulators.agent.AgentModel;
import edu.ncsu.simulators.agent.AgentModels;
import edu.ncsu.simulators.dialogs.ReferralChain;
import edu.ncsu.simulators.dialogs.ReferralChainDAG;

import edu.ncsu.simulators.util.ErrorHandler;
import edu.ncsu.simulators.util.Heuristics;
import edu.ncsu.simulators.util.BuildLattice;
import edu.ncsu.simulators.vectors.Answer;
import edu.ncsu.simulators.vectors.Expertise;
import edu.ncsu.simulators.vectors.Interests;
import edu.ncsu.simulators.vectors.Query;
import edu.ncsu.simulators.vectors.LatticeExpertise;
import edu.ncsu.simulators.vectors.Sociability;
import edu.ncsu.simulators.vectors.ScalarSociability;
import edu.ncsu.simulators.vectors.lattice.*;

import java.io.FileWriter;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.HashSet;
import java.util.Vector;

/**
 *  
 *
 * @author <a href="mailto:pyolum@eos.ncsu.edu">Pinar Yolum</a>
 */

public class LearningPolicy_Lattice implements LearningPolicy {

    private static final double BAD_ANSWER = 0.0;
    private static final double GOOD_ANSWER = 1.0;
    private static final double NO_ANSWER = 2.0;

    private FileWriter fwLog = null;
    private Hashtable actualValues = new Hashtable();

    private int goodAnswerCount =0;

    private int badAnswerCount=0;

    private int noAnswerCount=0;

    public LearningPolicy_Lattice(){
	//defaultValues();
	actualValues = BuildLattice.assignProviders(50 , 7 ,-1);
	//System.out.println(actualValues);
	
    }

    public void defaultValues() {
	Vector type0 = new Vector(); // Can do only the first three
	Vector type1 = new Vector(); // Can do only the first three
	Vector type2 = new Vector(); // Can do the first six
	Vector type3 = new Vector(); // CAn do the first eight
	Vector type4 = new Vector(); //Can do all
	Vector type5 = new Vector(); //Can do all
	Vector type6 = new Vector(); //Can do all
	Vector type7 = new Vector(); //Can do all
	Vector type8 = new Vector(); //Can do all

	type0.addElement(new Integer(1));
	type0.addElement(new Integer(0));
	type0.addElement(new Integer(0));
	type0.addElement(new Integer(0));
	type0.addElement(new Integer(0));
	type0.addElement(new Integer(0));
	type0.addElement(new Integer(0));
	type0.addElement(new Integer(0));
	type0.addElement(new Integer(0));
	actualValues.put (new Double(100), type0);
	actualValues.put (new Double(101), type0);
	actualValues.put (new Double(102), type0);
	actualValues.put (new Double(103), type0);
	actualValues.put (new Double(104), type0);
	actualValues.put (new Double(105), type0);
	actualValues.put (new Double(106), type0);
	actualValues.put (new Double(107), type0);
	actualValues.put (new Double(108), type0);
	actualValues.put (new Double(109), type0);
	actualValues.put (new Double(110), type0);


	type1.addElement(new Integer(1));
	type1.addElement(new Integer(1));
	type1.addElement(new Integer(0));
	type1.addElement(new Integer(0));
	type1.addElement(new Integer(0));
	type1.addElement(new Integer(0));
	type1.addElement(new Integer(0));
	type1.addElement(new Integer(0));
	type1.addElement(new Integer(0));

	actualValues.put (new Double(111), type1);
	actualValues.put (new Double(112), type1);
	actualValues.put (new Double(113), type1);

	type2.addElement(new Integer(1));
	type2.addElement(new Integer(0));
	type2.addElement(new Integer(1));
	type2.addElement(new Integer(0));
	type2.addElement(new Integer(0));
	type2.addElement(new Integer(0));
	type2.addElement(new Integer(0));
	type2.addElement(new Integer(0));
	type2.addElement(new Integer(0));

	actualValues.put (new Double(114), type2);
	actualValues.put (new Double(115), type2);
	actualValues.put (new Double(116), type2);

	type3.addElement(new Integer(1));
	type3.addElement(new Integer(1));
	type3.addElement(new Integer(0));
	type3.addElement(new Integer(1));
	type3.addElement(new Integer(0));
	type3.addElement(new Integer(0));
	type3.addElement(new Integer(0));
	type3.addElement(new Integer(0));
	type3.addElement(new Integer(0));

	actualValues.put (new Double(117), type3);
	actualValues.put (new Double(118), type3);
	actualValues.put (new Double(119), type3);


	type4.addElement(new Integer(1));
	type4.addElement(new Integer(1));
	type4.addElement(new Integer(1));
	type4.addElement(new Integer(0));
	type4.addElement(new Integer(1));
	type4.addElement(new Integer(0));
	type4.addElement(new Integer(0));
	type4.addElement(new Integer(0));
	type4.addElement(new Integer(0));

	actualValues.put (new Double(120), type4);
	actualValues.put (new Double(121), type4);
	actualValues.put (new Double(122), type4);


	type5.addElement(new Integer(1));
	type5.addElement(new Integer(0));
	type5.addElement(new Integer(1));
	type5.addElement(new Integer(0));
	type5.addElement(new Integer(0));
	type5.addElement(new Integer(1));
	type5.addElement(new Integer(0));
	type5.addElement(new Integer(0));
	type5.addElement(new Integer(0));

	actualValues.put (new Double(123), type5);
	actualValues.put (new Double(124), type5);
	actualValues.put (new Double(125), type5);

	type6.addElement(new Integer(1));
	type6.addElement(new Integer(1));
	type6.addElement(new Integer(1));
	type6.addElement(new Integer(1));
	type6.addElement(new Integer(1));
	type6.addElement(new Integer(0));
	type6.addElement(new Integer(1));
	type6.addElement(new Integer(0));
	type6.addElement(new Integer(0));
	actualValues.put (new Double(126), type6);
	actualValues.put (new Double(127), type6);

	type7.addElement(new Integer(1));
	type7.addElement(new Integer(1));
	type7.addElement(new Integer(1));
	type7.addElement(new Integer(0));
	type7.addElement(new Integer(1));
	type7.addElement(new Integer(1));
	type7.addElement(new Integer(0));
	type7.addElement(new Integer(1));
	type7.addElement(new Integer(0));
	actualValues.put (new Double(128), type7);
	actualValues.put (new Double(129), type7);

	type8.addElement(new Integer(1));
	type8.addElement(new Integer(1));
	type8.addElement(new Integer(1));
	type8.addElement(new Integer(1));
	type8.addElement(new Integer(1));
	type8.addElement(new Integer(1));
	type8.addElement(new Integer(1));
	type8.addElement(new Integer(1));
	type8.addElement(new Integer(1));
	actualValues.put (new Double(130), type8); //modified for the no 8 experiments
	actualValues.put (new Double(131), type8);
    }

    public void learnFromQuestion( AgentModel modelQuestioner, Query query ){
    }
    
    public void setFileWriter( FileWriter fw ) {
	this.fwLog = fw;
    }	

    public void learnFromResponses( ReferralChain referralChain, AgentModels agentModels, Expertise expertiseOfAskingAgent){

	goodAnswerCount = 0;

	badAnswerCount = 0;

	noAnswerCount = 0;

	Query query = referralChain.getQuery();
     
	Integer previousRole = new Integer(-1);
	Integer newRole = new Integer(((Double)query.elementAt(0)).intValue());

	Hashtable hashAnswers = referralChain.getAnswers();
	HashSet answers = new HashSet();

	Enumeration enumAgentIDs = hashAnswers.keys();
	Enumeration enumAnswers = hashAnswers.elements();

	LatticeExpertise l = (LatticeExpertise)expertiseOfAskingAgent;

	WeightMatrix weights = l.getWeights();
	Hashtable promotionTable = l.getPromotionTable();

	while ( enumAgentIDs.hasMoreElements() ){
	    Integer answeringAgentID = (Integer)enumAgentIDs.nextElement();

	    Answer answer = (Answer)enumAnswers.nextElement();

	    AgentModel modelAnsweringAgent = agentModels.getModel( answeringAgentID );

	    double grade = evaluateAnswer( query, answer );
	    
	    adjustCounts(grade);

	    //Update the questioners own lattice after evaluating the answer	    
	    if ((grade != NO_ANSWER) && (!answers.contains(answer))){
		answers.add(answer);
		int answerValue = ((Double)answer.elementAt(0)).intValue();
		l.updateProviders (newRole.intValue(), answerValue, grade);
		
	    } // end of if ()
	    

	    //Update the answerer's model
	    updateExpertise( modelAnsweringAgent, grade, answer, query );


	    int generationsRemoved = 0;

	    Enumeration enumGenerations = referralChain.getAncestorsByGeneration( answeringAgentID.intValue() );

	    while ( enumGenerations.hasMoreElements()) {

		generationsRemoved++;
		Enumeration enumAncestors = (Enumeration)enumGenerations.nextElement();
		while( enumAncestors.hasMoreElements() ) {

		    Integer ancestorID = (Integer)enumAncestors.nextElement();
		    AgentModel modelAncestor = agentModels.getModel( ancestorID );
		    //double oldSociability = modelAncestor.getSociability();

		    updateSociability( modelAncestor, grade, generationsRemoved );
		    //double newSociability = modelAncestor.getSociability();
		} //end while
	    } //end while
	} //end while

	try {
	    if  (fwLog != null) {
		fwLog.write( "GOOD " + goodAnswerCount + '\r' + '\n' );
		fwLog.write( "BAD " + badAnswerCount + '\r' + '\n' );
		fwLog.write( "NO " + noAnswerCount + '\r' + '\n' );
	    }
	}
	catch ( Exception e ) {
	    e.printStackTrace();
	}
    }
    /**
     * <code>adjustCounts</code> increases the number of good answers,
     * bad answers, or no answers.
     *
     * @param evaluation a <code>double</code> value for grade 
     */

    public void adjustCounts (double evaluation) {
	if (evaluation == NO_ANSWER) {
    	    noAnswerCount++;
	} // end of if ()
	else if (evaluation == GOOD_ANSWER) {
	    goodAnswerCount++;
	    
	}
	else badAnswerCount++;
	// end of if ()else
    }

    
    /**
     * Evaluates the specified answer to the specified question, and returns
     * a grade.  The grade is no answer if there were no answers or it is a 
     * good answer by default for now.
     *
     * @param query a <code>Query</code> value
     * @param answer an <code>Answer</code> value
     * @return a <code>double</code> value
     */
    public double evaluateAnswer( Query query, Answer answer ){

	double evaluation = -1;
	int roleId = ((Double)query.elementAt(0)).intValue();

	if ((answer == Constants.NULL_ANSWER) || (answer == null)) 
	    evaluation = NO_ANSWER;

	else {

	    Double providerId = (Double)answer.elementAt(0);
	    Vector  v = (Vector)actualValues.get(providerId);
	    Integer actualPerformance = new Integer(0);

	    if ( v != null)
		    actualPerformance = (Integer)v.elementAt(roleId);

	    if (actualPerformance.intValue()< Constants.SIMILARITY_THRESHOLD_FOR_EVALUATING_ANSWER){
		    evaluation = BAD_ANSWER;
	    }
	    else {
		    evaluation = GOOD_ANSWER;
	    }
	}
	return evaluation;

    }


    //======================================================================== 

    /**
     * Updates the expertise of the specified agent model based on the
     * specified answer that it provided to the specified query.
     */

    private void updateExpertise( AgentModel answererModel, double grade, Answer answer, Query query )
    {
	
	Expertise currentExpertise = answererModel.getExpertise();
	//	Expertise newExpertise = new LatticeExpertise();
	double newValue = 0.0;
	Double currentExpertiseComponent = null;

	Double newExpertiseComponent = null;

	if ( (grade == GOOD_ANSWER) || (grade == BAD_ANSWER) ){

	    for ( int i=0; i<currentExpertise.size(); i++ )
		//For the role that the answer came for:
		if (i == ((Double)query.elementAt(0)).intValue()) {
		    newValue = ((Double)currentExpertise.elementAt(i)).doubleValue();
		    if ( grade == GOOD_ANSWER ) {
			newValue += Constants.ALPHA * newValue;	//alpha is a small value
		    } // end of if ()
		    else {
			newValue -= Constants.ALPHA * newValue;	
		    } // end of else
		    
		    currentExpertise.set(i, new Double(newValue));
		}
	}

	else if ( grade == NO_ANSWER ){

	    for ( int i=0; i<currentExpertise.size(); i++ ){
		       
		currentExpertiseComponent = (Double)currentExpertise.elementAt( i );
		newValue = (1-Constants.PERCENTAGE_DECAY_FOR_NO_ANSWER) * currentExpertiseComponent.doubleValue();
		currentExpertise.set(i, new Double(newValue) );
	    }
	}

	else
	    ErrorHandler.exit( "Attempting to update expertise based on an answer with an unknown grade type" );

	answererModel.setExpertise(currentExpertise );


    }

    /**
     * Updates the sociability of the specified agent based on the assumption
     * that is was separated by <generationsRemoved> referrals from an agent
     * that gave an answer which evaluated to <grade>
     */

    private void updateSociability( AgentModel modelAncestor, double grade, int generationsRemoved ){
	
	double newSociability = ((ScalarSociability)modelAncestor.getSociability()).getSociability();

	if ( grade == GOOD_ANSWER )
	    newSociability += (((1-newSociability)*Constants.BETA)/generationsRemoved);

        else if ( (grade == BAD_ANSWER) || (grade == NO_ANSWER) )
            newSociability -= ((newSociability*Constants.BETA)/generationsRemoved);

        else
	    ErrorHandler.exit( "Attempting to update sociability based on an answer with an unknown grade type" );

	newSociability = Math.min(1, newSociability);
	newSociability = Math.max(0, newSociability);

	modelAncestor.setSociability(new ScalarSociability( newSociability) );

    }
}

