package edu.ncsu.simulators.policies;

import edu.ncsu.simulators.vectors.Query;
import edu.ncsu.simulators.vectors.Expertise;
import edu.ncsu.simulators.vectors.Interests;
import edu.ncsu.simulators.Constants;
import java.util.*;
/**
 * QueryGenerationPolicy_Standard.java
 *
 *
 * Created: Tue Aug 20 17:56:09 2002
 *
 * @author <a href="mailto:pyolum@eos.ncsu.edu">Pinar Yolum</a>
 * @version
 */

public class QueryGenerationPolicy_Standard implements QueryGenerationPolicy{
    public QueryGenerationPolicy_Standard (){
	
    }

    /**
     * This is the default policy where each agent generates a query
     * based on its interest vector.
     *
     * @param expertise an <code>Expertise</code> value
     * @param interests an <code>Interests</code> value
     * @return a <code>Query</code> value 
     */
    public Query generateQuery(Expertise expertise, Interests interests, Random random) {

	Query newQuery = new Query();		

	double percentage = 0;
	double sign = 0;
	double oldInterests = 0;
	double newInterests = 0;

	Iterator iteratorInterestsComponents = interests.iterator();

	// generate the query based on this agent's interests

	while( iteratorInterestsComponents.hasNext()) {

	    percentage = random.nextDouble() * Constants.MAXIMUM_PERCENTAGE_PERTURBATION_FOR_GENERATING_QUERY;			
	    sign = random.nextDouble();

	    if (sign < 0.5)
		sign = -1;
	    else
		sign = 1;

	    oldInterests = ((Double)iteratorInterestsComponents.next()).doubleValue();		
	    newInterests = oldInterests + (sign * percentage * oldInterests);

	    // the new value can not be less than 0, or greater than 1			
	    
	    newInterests = Math.max( 0, newInterests );
	    newInterests = Math.min( 1, newInterests );

	    newQuery.add( new Double(newInterests) );

	}		
	return newQuery;

    }	
}// QueryGenerationPolicy_Standard
