/*
 * ReferralPolicyBestNeighbor.java
 *
 * Created on November 5, 2001, 1:22 PM
 */

package edu.ncsu.simulators.policies;

/**
 *
 * @author  pirnal
 * @version 
 */
import edu.ncsu.simulators.Constants;
import edu.ncsu.simulators.agent.AgentModel;
import edu.ncsu.simulators.agent.AgentModels;
import edu.ncsu.simulators.agent.Neighbors;
import edu.ncsu.simulators.util.Heuristics;
import edu.ncsu.simulators.vectors.Query;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;
import java.util.Collections;

public class ReferringPolicy_BestNeighbor implements ReferringPolicy
{
	//=======================================================================
	
	public ReferringPolicy_BestNeighbor()
	{
	}
	
	//=======================================================================
	/**
	 * @param query the query that the questioning agent wants to send
	 * @param neighbors all of the neighbors whom the questioning agent can consider sending the query to
	 * @param agentModels all of the agent models that the questioning agent maintains
	 *
	 * @return a collection of neighbors whom the questioning agent should send
	 * the query to (this will be a subset of the agents specified in the neighbors parameter)
	 */ 
	public Iterator getNeighborsToReferTo(AgentModel modelQuestioner,  Query query, Neighbors neighbors, AgentModels agentModels, int myId )
	{
		return getBestNeighbor( query, neighbors, agentModels);
	}	
	
	//=======================================================================
	/**
	 * Returns an enumeration of the ID's of those neighbors who are 
	 * thought to be knowledgable about this query
	 */
	private Iterator getBestNeighbor( Query query, Neighbors neighbors, AgentModels agentModels)
	{	
		Iterator iteratorNeighbors = neighbors.iterator();
		Vector v = new Vector();
		Integer bestNeighbor = new Integer(0);
		double bestValue = 0.0;
		double current = -1.0;
                
		while( iteratorNeighbors.hasNext() )
		{
			Integer neighborID = (Integer)iteratorNeighbors.next();		
			AgentModel neighborModel = agentModels.getModel( neighborID );		
			current = Heuristics.calculateSimilarity(query, neighborModel.getExpertise());
			if (current > bestValue) {
			  bestValue = current;
			  bestNeighbor = neighborID;
			}

		}
		v.add(bestNeighbor);
		return v.iterator();
	}	
	
	//=======================================================================
}
