//==========================================================================
//	This source code is part of the Agent Simulation Test Bed system which
//	was developed for simulating interaction among agents who are attempting 
//	to acquire information from each other and learn about each other. 
//
//	The system was created at North Carolina State Univeristy, 
//	Department of Computer Science between September and December 2001. 
//	It was implemented by Michael J. Suralik based on the specifications 
//	developed by Pinar Yolum and Bin Yu, who were conducting research under 
//   the guidance of Dr. Munindar Singh. 
//==========================================================================

package edu.ncsu.simulators.policies;

import edu.ncsu.simulators.Constants;
import edu.ncsu.simulators.agent.AgentModel;
import edu.ncsu.simulators.agent.AgentModels;
import edu.ncsu.simulators.agent.Neighbors;
import edu.ncsu.simulators.policies.Heuristics;
import edu.ncsu.simulators.vectors.Query;

import java.util.Hashtable;
import java.util.Iterator;

/**
 * This "standard" policy provides a very straight forward implementation of a
 * ReferringPolicy.
 *
 * @author Mike Suralik
 */
public class ReferringPolicy_Standard implements ReferringPolicy
{
	//=======================================================================
	
	public ReferringPolicy_Standard()
	{
	}
	
	//=======================================================================
	/**
	 * This implementation of the referral policy ignores the model of the 
	 * questioner, and treats all questioning agents equally.  The referring
	 * agent will return the IDs of those neighbors whose models indicate
	 * that their expertise is similiar to the query.
	 */ 
	public Iterator getNeighborsToReferTo( AgentModel modelQuestioner, Query query, Neighbors neighbors, AgentModels agentModels, int myId )
	{
		return getNeighborsWithSufficientSimilarity( query, neighbors, agentModels, Constants.SIMILARITY_THRESHOLD_FOR_REFERING_QUESTIONER_TO_NEIGHBOR );
	}	
	
	//=======================================================================
	/**
	 * Returns an enumeration of the ID's of those neighbors who are 
	 * thought to be knowledgable about this query
	 */
	private Iterator getNeighborsWithSufficientSimilarity( Query query, Neighbors neighbors, AgentModels agentModels, double similarityThreshold )
	{	
		Hashtable hashKnowledgeableNeighbors = new Hashtable();
		Iterator iteratorNeighbors = neighbors.iterator();
		
		while( iteratorNeighbors.hasNext() )
		{
			Integer neighborID = (Integer)iteratorNeighbors.next();		
			AgentModel neighborModel = agentModels.getModel( neighborID );			
			
			if ( Heuristics.areSufficientlySimilar( query, neighborModel.getExpertise(), similarityThreshold ) )
			{
				hashKnowledgeableNeighbors.put( neighborID, neighborID );
			}
		}
		
		return hashKnowledgeableNeighbors.values().iterator();
	}	
	
	//=======================================================================
	
}
