
package edu.ncsu.simulators.tests;

import edu.ncsu.simulators.agent.*;
import edu.ncsu.simulators.service.*;
import edu.ncsu.simulators.classification.*;
import edu.ncsu.simulators.evaluation.*;
import edu.ncsu.simulators.evaluation.evaluators.*;
import edu.ncsu.simulators.evaluation.results.*;

public class WeightedEvaluatorTest {
	public static HierarchicalCategory root,travel,air,car;	
	public static HierarchicalClassification classification;
	public static void main(String arg[]){
		System.out.println("Hello world");
		initialize();
		test1(arg);
		System.out.println("Test 1 passed");
		test2(arg);
		System.out.println("Test 2 passed");		
	}

	public static void initialize() {
		root = new HierarchicalCategory("Commerce");
		travel = new HierarchicalCategory("Travel");
		air = new HierarchicalCategory("Air");
		car = new HierarchicalCategory("Car");
		
		travel.addParent(root);
		air.addParent(travel);
		car.addParent(travel);
		
		classification = new HierarchicalClassification(root);				
		
	}
	
	public static void test1(String arg[]){
		/* Create services and initialize them
		 */
		ServiceProvider a,b,c,d;
		
		a = new ServiceProvider(air,"A");
		b = new ServiceProvider(air,"B");
		c = new ServiceProvider(air,"C");
		d = new ServiceProvider(air,"D");
		
		/* Create agents and set their agent models
		 */
		RatingAgentModel N1,N2,N3;
		AgentModels models = new AgentModels();
		
		RatingAgent myAgent = new RatingAgent();

		N1 = new RatingAgentModel(classification);
		N2 = new RatingAgentModel(classification);
		N3 = new RatingAgentModel(classification);
		N1.setRating(air,new Rating(new FloatValue(0.2)));		
		N2.setRating(air,new Rating(new FloatValue(0.3)));
		N3.setRating(air,new Rating(new FloatValue(0.4)));

		models.addModel(N1);
		models.addModel(N2);
		models.addModel(N3);
		
		myAgent.setAgentModels(models);
		
		/*Build the score table
		 */
		 Score n1a,n1b,n1c,n2a,n2b,n2c,n2d,n3b,n3c,n3d;
		 ScoreTable table = new ScoreTable();
		 
		 n1a = new Score(N1.getID(), a, new FloatValue(0.3));
		 n1b = new Score(N1.getID(), b, new FloatValue(0.5));
		 n1c = new Score(N1.getID(), c, new FloatValue(0.1));

		 n2a = new Score(N2.getID(), a, new FloatValue(0.2));
		 n2b = new Score(N2.getID(), b, new FloatValue(0.4));
		 n2c = new Score(N2.getID(), c, new FloatValue(0.1));
		 n2d = new Score(N2.getID(), d, new FloatValue(0.5));

		 n3b = new Score(N3.getID(), b, new FloatValue(0.3));
		 n3c = new Score(N3.getID(), c, new FloatValue(0.2));
		 n3d = new Score(N3.getID(), d, new FloatValue(0.1));

		table.addScore(n1a);
		table.addScore(n1b);
		table.addScore(n1c);
		table.addScore(n2a);
		table.addScore(n2b);
		table.addScore(n2c);
		table.addScore(n2d);
		table.addScore(n3b);
		table.addScore(n3c);
		table.addScore(n3d);
		
		/* Get this table evaluated by the weighted evaluator
		 */
		 
		 WeightedEvaluator eval = new WeightedEvaluator(myAgent);
		 Evaluation evaluation = eval.evaluate(table);
		 //System.out.println(((ServiceEvaluation)evaluation).toString());
		 System.out.println(evaluation.toString());
	}
	
	public static void test2(String arg[]){
		/* Create services and initialize them
		 */
		ServiceProvider a,b,c,d;
		
		a = new ServiceProvider(travel,"A");
		b = new ServiceProvider(travel,"B");
		c = new ServiceProvider(travel,"C");
		d = new ServiceProvider(travel,"D");
		
		/* Create agents and set their agent models
		 */
		RatingAgentModel N1,N2,N3;
		AgentModels models = new AgentModels();
		
		RatingAgent myAgent = new RatingAgent();

		N1 = new RatingAgentModel(classification);
		N2 = new RatingAgentModel(classification);
		N3 = new RatingAgentModel(classification);
		N1.setRating(air,new Rating(new FloatValue(0.2)));		
		N2.setRating(air,new Rating(new FloatValue(0.3)));
		N3.setRating(air,new Rating(new FloatValue(0.4)));
		N1.setRating(car,new Rating(new FloatValue(0.2)));		
		N2.setRating(car,new Rating(new FloatValue(0.3)));
		N3.setRating(car,new Rating(new FloatValue(0.4)));

		models.addModel(N1);
		models.addModel(N2);
		models.addModel(N3);
		
		myAgent.setAgentModels(models);
		
		/*Build the score table
		 */
		 Score n1a,n1b,n1c,n2a,n2b,n2c,n2d,n3b,n3c,n3d;
		 ScoreTable table = new ScoreTable();
		 
		 n1a = new Score(N1.getID(), a, new FloatValue(0.3));
		 n1b = new Score(N1.getID(), b, new FloatValue(0.5));
		 n1c = new Score(N1.getID(), c, new FloatValue(0.1));

		 n2a = new Score(N2.getID(), a, new FloatValue(0.2));
		 n2b = new Score(N2.getID(), b, new FloatValue(0.4));
		 n2c = new Score(N2.getID(), c, new FloatValue(0.1));
		 n2d = new Score(N2.getID(), d, new FloatValue(0.5));

		 n3b = new Score(N3.getID(), b, new FloatValue(0.3));
		 n3c = new Score(N3.getID(), c, new FloatValue(0.2));
		 n3d = new Score(N3.getID(), d, new FloatValue(0.1));

		table.addScore(n1a);
		table.addScore(n1b);
		table.addScore(n1c);
		table.addScore(n2a);
		table.addScore(n2b);
		table.addScore(n2c);
		table.addScore(n2d);
		table.addScore(n3b);
		table.addScore(n3c);
		table.addScore(n3d);
		
		/* Get this table evaluated by the weighted evaluator
		 */
		 
		 WeightedEvaluator eval = new WeightedEvaluator(myAgent);
		 Evaluation evaluation = eval.evaluate(table);
		 //System.out.println(((ServiceEvaluation)evaluation).toString());
		 System.out.println(evaluation.toString());
	}	

		
}
