package edu.ncsu.simulators.util;

import edu.ncsu.simulators.vectors.SimulationVector;
import edu.ncsu.simulators.vectors.lattice.*;

import java.util.Enumeration;
import java.util.*;

/**
 * BuildLattice.java
 *
 *
 * Created: Wed Jan 29 20:24:03 2003
 *
 * @author <a href="mailto:pyolum@eos.ncsu.edu">Pinar Yolum</a>
 * @version
 */

public class BuildLattice {
    public BuildLattice (){
	
    }


    /**
     * Math package doesn't seem to have log base 10.  Which century
     * are we in?
     *
     * @param n a <code>double</code> value
     * @return a <code>double</code> value 
     */

    public static double log10(double n) {

	return Math.log(n)/Math.log(10);
    }

    /**
     * Calculate the total number of providers per level. Calculate the yintercept here.
     *
     * @param numberOfProviders an <code>int</code> value
     * @param numberOfLevels an <code>int</code> value
     * @param slope a <code>double</code> value
     * @return a <code>Vector</code> value
     */
    public static Vector distributeProviders(int numberOfProviders, int numberOfLevels, double slope) {

	double yIntercept = log10((new Integer(numberOfProviders)).doubleValue());
	return distributeProviders(numberOfProviders, numberOfLevels, slope,  yIntercept);
    }

    /**
     * Calculate the total number of providers per level given the yintercept
     *
     * @param numberOfProviders an <code>int</code> value
     * @param numberOfLevels an <code>int</code> value
     * @param slope a <code>double</code> value
     * @return a <code>Vector</code> value
     */
    public static Vector distributeProviders(int numberOfProviders, int numberOfLevels, double slope, double yIntercept) {

	Vector distribution = new Vector();
	double y = 0.0;

	for (int i=numberOfLevels; i>=1; i--) {

	    y = (slope * log10(i)) + yIntercept;
	    distribution.add(new Integer(Math.round(new Double(Math.pow(10,y)).floatValue())));
	} // end of for ()
	
	return distribution;
    }

    public static int totalServiceTypes(int numberOfLevels) {
	
	return ((numberOfLevels/2)+1)*((numberOfLevels/2)+1);
	
    }

    /**
     * Assign number of providers per service type
     *
     * @param numberOfProviders an <code>int</code> value
     * @param numberOfLevels an <code>int</code> value
     * @return an <code>int</code> value
     */
    public static Vector providersPerType(int numberOfProviders, int numberOfLevels, double slope) {
	
	int types = 0;
	int previous = 0;
	int current = 0; 
	int divide = 0; 
	Vector distribution = distributeProviders(numberOfProviders, numberOfLevels, slope);
	Vector services = new Vector();

	for (int i=0; i<numberOfLevels; i++) {
	    
	    types = Math.min(i+1, Math.abs(i+1-numberOfLevels)+1);
	    current = ((Integer)distribution.elementAt(i)).intValue();
	    divide = Math.max(0,(current - previous));

	    if ((divide%types) != 0)  {
		divide = types*((divide/types)+1);
	    } // end of if ()

	    //	    System.out.println("current " + current + " divide " + divide + " types " + types);
	    
	    previous = previous + divide; 

	    for (int j=0; j<types; j++ ) {
		services.add(new Integer(divide/types));
		//System.out.println("service " + j + " "  + divide/types);

	    } // end of for ()
	    
	}	    
	return services;
    }

    /**
     * combines two integer vectors/
     *
     * @param a a <code>Vector</code> value
     * @param b a <code>Vector</code> value
     * @return a <code>Vector</code> value
     */
    public static Vector union (Vector a, Vector b) {

	Vector v = null;

	if (a == null) {
	    v = (Vector) b.clone();
	    
	} // end of if ()
	else if (b == null) {
	    v=(Vector) a.clone();
	    
	} // end of if ()
	// Combine only if they are the same size
	else 
	     
	    if (a.size() == b.size()) {
		v = new Vector();
		//for each element put 1 if either a or b has a 1
		for (int i=0; i<a.size(); i++) {
		    if ((((Integer)a.elementAt(i)).intValue() == 1) || (((Integer)b.elementAt(i)).intValue() == 1)) {
			v.add(new Integer(1));
		    } // end of if ()
		    else {
			v.add(new Integer(0));
		    } // end of if ()else
		
		} // end of for ()

	    } // end of if ()
	
	return v;
    }

    /**
     * Assigns provider ids to service types
     *
     * @param numberOfProviders an <code>int</code> value
     * @param numberOfLevels an <code>int</code> value
     * @param slope an <code>int</code> value
     * @param types a <code>HashTable</code> value
     * @return a <code>Hashtable</code> value
     */
    public static Hashtable assignProviders(int numberOfProviders, int numberOfLevels, double  slope) {
	
	Hashtable values = new Hashtable();
	Hashtable types = createTypes(numberOfLevels);
	//	System.out.println(types);
	
	Vector temp = new Vector();
	Vector v  = providersPerType(numberOfProviders, numberOfLevels, slope);
	//100 is dummy, to make the provider ids big
	int currentProvider = (numberOfProviders-1)+100;
	int number = 0; 

	for (int j=0; j<v.size(); j++) {
	    
	    number = ((Integer)v.elementAt(j)).intValue();
	    //	    System.out.println("number is " + number + " size " + v.size() + " j " + j);

	    for (int i=0; i<number; i++) {
		
		temp = (Vector)types.get(new Integer((v.size()-j)-1));
		values.put(new Double(currentProvider), temp);
		currentProvider--;
	    }
			   
	} // end of while ()

	return values;
    }

    /**
     * Returns the number of types that are in the level where
     * servicetype is
     *
     * @param numberOfLevels an <code>int</code> value
     * @param servicetype an <code>int</code> value
     * @return an <code>int</code> value 
     */

    public static int sizeOfLevel(int numberOfLevels, int servicetype) {

	int total = 0;
	int level = 0; 
	boolean found = false;
	int i = 0;

	while (!found) {

	    level = Math.min(i+1, Math.abs(i+1-numberOfLevels)+1);
	    total += level;
	    i++;
	    if (servicetype < total) {
		found = true;
	    } // end of if ()

	} // end of while ()

	return (level);
	
    }

    /**
     * Dirty getaround.  Messy messy messy
     *
     * @param service an <code>int</code> value
     * @param numberOfLevels an <code>int</code> value
     * @return a <code>Vector</code> value
     */
    public static Vector createVector(int service, int numberOfLevels, Hashtable types) {

	int current = sizeOfLevel(numberOfLevels, service);

	int previous = sizeOfLevel(numberOfLevels, service-current);
 

	Vector v = new Vector ();

	if (Math.abs(current - previous)>1) {
	    previous ++;
	    
	} // end of if ()

	int left = service-current;
	int right = service-previous;

	if ( Math.abs(sizeOfLevel(numberOfLevels, left) - current) != 1 ) {
	    left = 0; 
	} 

	if ( Math.abs(sizeOfLevel(numberOfLevels, right) - current) != 1 ) {
	    right = 0; 
	} 

	v = union((Vector)types.get(new Integer(left)), (Vector)types.get(new Integer (right)));
	v.set(service, new Integer(1));

	return v;
	    
    }
    /**
     * Returns a vector with first element 1 and the rest 0s.
     *
     * @param numberOfServices an <code>int</code> value
     * @return a <code>Vector</code> value
     */
    public static Vector baseVector(int numberOfServices) {

	Vector v = new Vector();

	v.add(new Integer(1));
	for ( int j = 1; j<numberOfServices; j++) {
	    v.add(new Integer(0));
	} // end of for ()
	
	return v;
    }

    /**
     * Creates a service vector for each service type and put it in a
     * hashtable
     *
     * @param numberOfLevels an <code>int</code> value
     * @return a <code>Hashtable</code> value 
     */
    public static Hashtable createTypes(int numberOfLevels) {

	Hashtable types = new Hashtable();
	int numberOfServices= totalServiceTypes(numberOfLevels);

	Vector v = new Vector();
	types.put(new Integer(0), baseVector(numberOfServices));
	

	for (int i=1; i<numberOfServices; i++) {

	    v = createVector(i, numberOfLevels, types);
	    //	    System.out.println(" service " + i + " " + v);
	    
	    types.put(new Integer(i), v);

	} // end of for ()
	
	return types;
    }
    
}// BuildLattice
