package edu.ncsu.simulators.vectors.lattice;

/**
 * ProviderList.java
 *
 *
 * Created: Tue May  7 18:58:14 2002
 *
 * @author <a href="mailto:pyolum@eos.ncsu.edu">Pinar Yolum</a>
 * @version
 */

import java.util.*;
import net.n3.nanoxml.*;
import edu.ncsu.simulators.Constants;
import edu.ncsu.simulators.io.LogWriter;

public class ProviderList {

    /** Essentially a vector but the elements can only be  providernodes 
     *  so that we don't have to cast every get 
     */
    protected Vector v = new Vector();
        
    protected Integer roleId = new Integer(-1);

    public ProviderList (){
	
    }

    public ProviderList (Integer i){
	roleId = i;
    }
    
    

    /**
     * Get the value of roleId.
     * @return value of roleId.
     */
    public Integer getRoleId() {
	return roleId;
    }
    
    /**
     * Set the value of roleId.
     * @param v  Value to assign to roleId.
     */
    public void setRoleId(Integer  v) {
	this.roleId = v;
    }
    

    /** Adds a new ProviderListNode */
    public void add(ProviderListNode x){
        v.add(x);
    }


    /**
     * Updates the list of ratings for an agent 
     *
     * @param agentId an <code>int</code> value
     * @param grade a <code>double</code> value
     */
    public void update(int agentId, double grade, boolean promoted){
	Iterator i = v.iterator();
	ProviderListNode pNode = new ProviderListNode(agentId);
	boolean found = false;
	Object temp = null;

	if (!i.hasNext()) {
	    // This agent does not know any providers. 
	    add(new ProviderListNode(agentId, new Double(grade), promoted));	    
	} // end of if ()
	else {
	    //If not check if there are ratings for the same provider
	    while (i.hasNext() && !found){
		//		temp = (ProviderListNode)i.next();
		//if (temp != null){
		pNode = (ProviderListNode)i.next(); //temp;
		if (pNode.getProviderId() == agentId){
			pNode.addRating(new Double (grade));
			found = true;
			//    }
		}
	    }
	    if (!found) {
		    add(new ProviderListNode(agentId, new Double(grade), promoted));
		    found = true;
	    } // end of if ()

	} // end of if ()else
	

    }

    public ProviderListNode elementAt(int n){
        return ((ProviderListNode)v.get(n));
    }

    public int size(){
        return v.size();
    }

    public void clear(){
        v.clear();
    }

    public Iterator iterator(){
        return v.iterator();
    }


    public void toXML(IXMLElement xmlProviderList) {
	try {	
	    // Set all the attributes
	    xmlProviderList.setAttribute(Constants.XML_ATTRIBUTE_ID, this.roleId.toString());
	    Iterator i = iterator();
	    
	    // Set all the child elements
	    while (i.hasNext()){
		IXMLElement xmlProviderListNode = xmlProviderList.createElement(Constants.XML_TAG_PROVIDER_LIST_NODE);
		((ProviderListNode)i.next()).toXML( xmlProviderListNode );	
		xmlProviderList.addChild( xmlProviderListNode );
	    }
	}
	catch(Exception e){
	    e.printStackTrace();
	}				
    }	

    public static ProviderList parse( IXMLElement xmlProviders ){

	ProviderList providers = new ProviderList();

	Vector vecChildElements = xmlProviders.getChildren();

	Properties attributes = xmlProviders.getAttributes();
	providers.setRoleId(Integer.valueOf(attributes.getProperty(Constants.XML_ATTRIBUTE_ID)));	 
	
	Enumeration enumChildElements = vecChildElements.elements();


	try{

	    while( enumChildElements.hasMoreElements()){

		IXMLElement xmlChildElement = (IXMLElement)enumChildElements.nextElement();
		
		if ( xmlChildElement.getName().equalsIgnoreCase(Constants.XML_TAG_PROVIDER_LIST_NODE)){
		    //System.out.println("ProviderListNode");
		    providers.add(ProviderListNode.parse( xmlChildElement));
		}
		else
		    System.exit(1);//Change this to error handler
	    }
	}
	catch( Exception e) {
	    e.printStackTrace();
	}			

	return providers;

    }

}// ProviderList





