package edu.ncsu.simulators.vectors.lattice;

import net.n3.nanoxml.*;
import java.util.*;
import java.io.*;
import edu.ncsu.simulators.Constants;
import edu.ncsu.simulators.io.LogWriter;
import edu.ncsu.simulators.util.Statistics;
import java.util.Random;
/**
 * WeightMatrix.java
 *
 *
 * Created: Tue May  7 19:30:02 2002
 *
 * @author <a href="mailto:pyolum@eos.ncsu.edu">Pinar Yolum</a>
 * @version
 */

public class WeightMatrix {
    
    protected double[][] weights;
    protected int dimension = 3;
    protected Hashtable pointAt = new Hashtable();

    private static final char DELIMITER = ' ';

    public WeightMatrix () {
    }
    public WeightMatrix (int numberOfRoles) {
	this.dimension = numberOfRoles;
	weights = new double[numberOfRoles][numberOfRoles];
	initialize();
    }

    public void initialize(){
	for (int i=0; i<dimension; i++)
	    	for (int j=0; j<dimension; j++)
		    weights[i][j] = -1;
    }

    /**
     * Describe <code>updateWeight</code> method here.
     *
     * @param i an <code>int</code> value
     * @param j an <code>int</code> value
     * @param newWeight a <code>double</code> value
     */
    public void updateWeight(int i, int j, double newWeight){
	weights[i][j] = newWeight;
    }

    public double find(int i, int j){
	return weights[i][j];
    }


    /**
     *  Find which nodes in point to node.  After finding it, put
     *  it in a hashtable so that successive accesses get it from
     *  the hashtable.
     *
     *
     * @param node an <code>int</code> value
     * @return a <code>HashSet</code> value
     */
    public HashSet pointTo(int node) {

	HashSet v = (HashSet)pointAt.get(new Integer(node));
	if (v == null){
	    v = new HashSet();
	    for (int i=0; i<dimension; i++)
		if (this.weights[i][node]>-1) v.add(new Integer(i));

	    pointAt.put(new Integer(node), v);
	}
	return v;    
    }


	
    /**
     * Describe <code>toPreciseString</code> method here.
     *
     * @return a <code>String</code> value
     */
    public String toPreciseString()
    {	
	StringBuffer sbuf = new StringBuffer();
	Double element = null;
	//Merge the array into a string
	for (int i=0; i<dimension; i++){
		for (int j=0; j<dimension; j++){
		    element = new Double(weights[i][j]);
		    sbuf.append(element.toString());
		    sbuf.append( DELIMITER );
		}
		//sbuf.append("\r \n");
	}

	return sbuf.toString();
    }
    
     //========================================================================	
	
    public final void toXML(IXMLElement xmlMatrix) {
	xmlMatrix.setContent( this.toPreciseString() );
    }	
    
    /**
     * Generates the weights of the matrix randomly.
     *
     * @param dimension an <code>int</code> value
     * @return a <code>WeightMatrix</code> value
     */
    public static WeightMatrix generate(int dimension) {

	WeightMatrix w = new WeightMatrix(dimension);

	 Random random = Statistics.getRandomNumberGenerator();
	for (int i=0; i<dimension; i++)
	    for (int j=0; j<dimension; j++) {
		double weight  = random.nextDouble();
		w.updateWeight(i,j,weight);

	    }
	return w;
    }

    static final Comparator edgeComp = new Comparator() {
      public int  compare(Object obj1, Object obj2)
                {
                        double i1 = ((Double)((Vector)obj1).elementAt(2)).doubleValue();
                        double i2 = ((Double)((Vector)obj2).elementAt(2)).doubleValue();
			int diff = 0;
			if ((i1-i2) > 0) {
			    diff = 1;
			} // end of if ()
			
                        return diff;
                }
	};	    

    /**
     * Keeps the top n weights in the matrix and resets the rest to -1.
     *
     * @param w a <code>WeightMatrix</code> value
     * @param top an <code>int</code> value
     * @param dimension an <code>int</code> value
     * @return a <code>WeightMatrix</code> value
     */
    public static WeightMatrix topN(WeightMatrix w, int top, int dimension) {
	
	Vector allEdges = new Vector();

	Vector oneEdge = new Vector();
	for (int i=0; i<dimension; i++) {
	    for (int j=0; j<dimension; j++){
		oneEdge = new Vector();
		oneEdge.add(new Integer(i));
		oneEdge.add(new Integer(j));
		oneEdge.add(new Double(w.weights[i][j]));
		allEdges.add(oneEdge);
		//oneEdge.clear();
	    }
	}

	//ArrayList edges = new ArrayList(allEdges);
	Collections.sort(allEdges, edgeComp);
	//System.out.println(((Vector)(allEdges.elementAt(0))).elementAt(2));
	//System.out.println(((Vector)(allEdges.elementAt(80))).elementAt(2));
	//Collections.reverse(allEdges);

	//dimension--;
	for (int k=0; k<((dimension*dimension)-top); k++) {
	    oneEdge = (Vector)(allEdges.elementAt(k));
	    //System.out.println(((Double)oneEdge.elementAt(2)).doubleValue());
	    oneEdge.setElementAt(new Double(-1), 2);
	    allEdges.setElementAt(oneEdge, k);
	    //System.out.println(k + " " + top + " " +dimension);
	}

	for ( int m=0; m<(dimension*dimension); m++) {
	    oneEdge = (Vector)(allEdges.elementAt(m));
	    w.updateWeight((((Integer)(oneEdge.elementAt(0))).intValue()), 
			   (((Integer)(oneEdge.elementAt(1))).intValue()), 
			   (((Double)(oneEdge.elementAt(2))).doubleValue()));
	    
	} // end of for ()
	
	return w;

	
    }

    /**
     * Parse the weight matrix in the state file.
     *
     * @param xmlNode an <code>IXMLElement</code> value
     * @return a <code>WeightMatrix</code> value
     */
    public static WeightMatrix parse(IXMLElement xmlNode){
	
	StringReader reader = new StringReader( xmlNode.getContent() );
	StreamTokenizer tokenizer = new StreamTokenizer( reader );
	//tokenizer.whitespaceChars( DELIMITER, DELIMITER );			
	tokenizer.whitespaceChars(0, 32);			

	Properties attributes = xmlNode.getAttributes();
	int dimension = Integer.parseInt( attributes.getProperty(Constants.XML_ATTRIBUTE_DIMENSIONS) );	 
	WeightMatrix w = new WeightMatrix(dimension);
	

	try{			
	    
	    for (int i=0; i<dimension; i++)
		for (int j=0; j<dimension; j++){
		    tokenizer.nextToken();
		    //w.updateWeight(i,j,tokenizer.nval);
		}
	}
	
	catch ( Exception e ){
		e.printStackTrace();
	}
		
	w = WeightMatrix.generate(dimension);
	//w = WeightMatrix.topN(w, 12, dimension);

	return w;
    }
}// WeightMatrix


