package edu.ncsu.simulators.agent;

import edu.ncsu.simulators.classification.*;
import edu.ncsu.simulators.proxy.*;
import edu.ncsu.simulators.evaluation.*;
import edu.ncsu.simulators.evaluation.results.*;
import edu.ncsu.simulators.evaluation.results.ScoreTable;
import edu.ncsu.simulators.util.Statistics;
import edu.ncsu.simulators.SimulationContext;
import edu.ncsu.simulators.Constants;
import edu.ncsu.simulators.vectors.*;
import edu.ncsu.simulators.dialogs.UnprocessedReferrals;
import edu.ncsu.simulators.dialogs.Cache;
import edu.ncsu.simulators.io.LogWriter;

import javax.jms.*;
import java.util.*;
import java.io.File;
import java.io.FileWriter;

public class RatingAgent extends Agent{

    private Rating rating;
    private Service_proxy sp;
    private Queue_proxy queue;
    private Registry_proxy registry;

    public RatingAgent(int id) {
        super();
        initialize(id);
        sp = new Service_proxy();
        queue = new Queue_proxy();
        registry = new Registry_proxy();
    }

    public RatingAgent() {
        super();
        //initialize((int)Math.round(Math.random()*10000));
        id = (int)Math.round(Math.random()*10000);
        sp = new Service_proxy();
        queue = new Queue_proxy();
        registry = new Registry_proxy();
    }

    // copied from createAgent() function of the superclass.

    private void initialize(int id) {
		/** ------------ THE AGENT's FUNDAMENTAL ATTRIBUTES ---------- **/
		id = id;
		// Expertise, Interests, and Neighbors must be set using the
		// corresponding set methods.

		/** ------------ OTHER INFRASTRUCTURE ------------------------- **/

		random = Statistics.getRandomNumberGenerator();

		File fileCreditPolicyLog = new File( SimulationContext.getLogDir() + File.separator + Constants.AGENT_LOG_FILE_NAME_PREFIX + id + ".STATISTICS" );
		try
		{
			if ( Constants.OPTION_WRITE_AGENT_LOGS )
			{
				agentLogWriter = new LogWriter( SimulationContext.getLogDir() + File.separator + Constants.AGENT_LOG_FILE_NAME_PREFIX + getIDString( id ) + Constants.AGENT_LOG_FILE_NAME_SUFFIX  );
			}
			if ( Constants.OPTION_WRITE_AGENT_STATISTICS )
			{
				fwLearningPolicyLog = new FileWriter( fileCreditPolicyLog );
			}
		}
		catch( Exception e )
		{
			e.printStackTrace();
		}

		/** ------------ THE AGENT's MEMORY --------------------------- **/

		agentModels = new AgentModels();
		// IF NOT A NEW AGENT, AGENTMODELS MUST BE SET EXPLICITLY USING THE SET METHOD

		currentQuery = null;
		setAgentsWhomHaveBeenSentCurrentQuery = new HashSet();
		referralChain = null;
		unprocessedReferrals = new UnprocessedReferrals();
                cache = new Cache();

		/** ------------ THE AGENT's DECISION MAKING POLICIES -------- **/

		// All policies must be set using the corresponding set method.

		/** ------------ COMMUNICATION INFRASTRUCTURE ----------------- **/

		try
		{
			// Create the session
			queueSession = SimulationContext.getQueueConnection().createQueueSession( false, Session.AUTO_ACKNOWLEDGE );
			// Create sender (not associated with any particular queue)
			queueSender = queueSession.createSender( null );
			// Look up the destination
			Queue queueInboundMessages = (Queue)SimulationContext.getJNDIContext().lookup( getQueueName( id ) );
			// Create a subscriber
			queueReceiver = queueSession.createReceiver( queueInboundMessages );
			// Set the message listener, which is this class since we implement
			// the MessageListener interface
			queueReceiver.setMessageListener( this );
		}
		catch ( Exception e )
		{
			e.printStackTrace();
		}
	
    }

	public void setRating(Value v){
		rating.update(v);
	}
	
    public Rating getRating(){
        return rating;
    }

    public Vector findServices(Category c) {
        return registry.getServices(c);
    }

}
