/*
 * Adjacency.java
 *
 * Created on October 11, 2001, 12:06 PM
 */

package edu.ncsu.simulators.analysis;

import java.util.*;

/**
 *
 * @author  pirnal
 * @version 
 */

  /** This is the adjacency table implementation
   *  which is a hashtable that maps integers (node ids) to 
   *  treesets (neighbors of the node).
   */
public class Adjacency extends IntHashTable{


    /** Creates new Adjacency */
    public Adjacency() {
      // super();
    }

    /**
     * Returns the neighbors of node k
     *
     * @param k an <code>int</code> value
     * @return a <code>TreeSet</code> value
     */
    public TreeSet get(int k){
      TreeSet temp = (TreeSet)geth().get(new Integer(k));
      if (temp == null) temp = new TreeSet();
      return temp;
    }
    
    /**
     * Returns a new Adjacency with only elements in s
     *
     * @param s a <code>Set</code> value
     * @return an <code>IntHashTable</code> value
     */
    public Adjacency takePart(Set s) {
	
	Adjacency small = new Adjacency();
	int current =0;
	TreeSet newNeighbors = new TreeSet();

	Iterator i = s.iterator();
	
	while (i.hasNext()) {

	    current = ((Integer)i.next()).intValue();
	    newNeighbors = (TreeSet)this.get(current).clone();
	    Iterator j = newNeighbors.iterator();
	    
	    //Also adjust the neighbors so that only neighbors in s
	    //remain around.
	    while (j.hasNext()) {
		if (!s.contains(j.next())) {
		    j.remove();
		} // end of if ()
		
	    } // end of while ()

	    small.put(current, newNeighbors);
	    
	} // end of while ()
	
	return small;

    }
    /**
     * Returns how many elements of t are not in h
     *
     * @param t a <code>TreeSet</code> value
     * @param h a <code>HashSet</code> value
     * @return an <code>int</code> value
     */
    public int difference(Set t, Set h) {

	Iterator i = t.iterator();
	Integer temp = null;
	int offby = 0;

	while (i.hasNext()) {
	    temp = (Integer)i.next();
	    if (!h.contains(temp)) {
		offby++;
	    } // end of if ()
	    
	} // end of while ()
	return offby;
    }


    /**
     * Returns true if j is a neighbor of i 
     *
     * @param i an <code>int</code> value
     * @param j an <code>int</code> value
     * @return a <code>boolean</code> value
     */
    public boolean isNeighbor(int i, int j){
        if (i==j) return false;
         else return get(i).contains(new Integer(j));
    }
    public boolean isNeighbor(Integer i, Integer j){
        if (i==j) return false;
         else return get(i.intValue()).contains(j);
    }   
    
    public void print(){
        for (int i=0; i<size();i++){
            System.out.print(i + " " + get(i));
            System.out.println();
        }
    }

    /**
     * for pagerank calculations. When a node doesn't have an
     * outdegree, it creates edges to all the rest.
     *
     * @param n an <code>int</code> value
     * @return a <code>TreeSet</code> value 
     */

    public  TreeSet createFakeEdges(int n) {
	TreeSet t = new TreeSet();

	for (int i=0; i<n; i++) {
	    t.add (new Integer(i));
	    
	} // end of for ()
	
	return t;
    }

    public void fill() {

	for (int i=0; i<this.size(); i++) {
	    if (((TreeSet)at(i)).size()==0) {
		put(i, createFakeEdges(this.size()));
	    } // end of if ()
	    
	    
	} // end of for ()
	

    }


    /**
     * Returns the number of neighbors i has
     *
     * @param i an <code>int</code> value
     * @return an <code>int</code> value
     */
    public int numberofNeighbors(int i){
	return get(i).size();
   }
}
