package edu.ncsu.simulators.io;

/**
 * AgentParser.java
 *
 *
 * Created: Thu Jun  6 21:53:12 2002
 *
 * @author <a href="mailto:pyolum@eos.ncsu.edu">Pinar Yolum</a>
 * @version
 */

import edu.ncsu.simulators.agent.*;
import edu.ncsu.simulators.policies.*;
import edu.ncsu.simulators.vectors.*;
import edu.ncsu.simulators.vectors.lattice.*;
import edu.ncsu.simulators.Constants;
import net.n3.nanoxml.*;
import java.util.*;

public class AgentParser {
    public AgentParser (){
	
    }

    public static Agent parseServiceAgent ( IXMLElement xmlAgent, Expertise ex, Interests i ){


	Agent  agent = null;//new ServiceAgent();
	int agentID = -1;				

	Expertise expertise = ex;
	Interests interests = i;

	Neighbors neighbors = new Neighbors();			

	AgentModels agentModels = new AgentModels();	

	AskingPolicy p_asking = null;
	AnsweringPolicy p_answering = null;
	ReferringPolicy p_referring = null;
	LearningPolicy p_learning = null;
	ChangingNeighborsPolicy p_changingNeighbors = null;
                
	CachingPolicy p_cachingPolicy = new CachingPolicy_Standard();
	QueryGenerationPolicy p_query = new QueryGenerationPolicy_Standard();

	Double number = null;		
	try{
	// Parse all the attributes

	    Properties attributes = xmlAgent.getAttributes();
	    
	    agentID = Integer.parseInt( attributes.getProperty(Constants.XML_ATTRIBUTE_ID) );
	    //   System.out.println("AgentID" + agentID);
	    // Parse all the child elements

	    Vector vecChildElements = xmlAgent.getChildren();
	    Enumeration enumChildElements = vecChildElements.elements();
	    while( enumChildElements.hasMoreElements() ){

		IXMLElement xmlChildElement = (IXMLElement)enumChildElements.nextElement();
		// parse Lattice

		if ( xmlChildElement.getName().equalsIgnoreCase(Constants.XML_TAG_EXPERTISE) )
		  expertise = expertise.parse( xmlChildElement );

		// parse Interests

		else if ( xmlChildElement.getName().equalsIgnoreCase(Constants.XML_TAG_INTERESTS) )
		   interests = interests.parse( xmlChildElement );

		// parse Neighbors

		else if ( xmlChildElement.getName().equalsIgnoreCase(Constants.XML_TAG_NEIGHBORS) )
			neighbors = Neighbors.parse( xmlChildElement );

		// parse Agent Model

		else if ( xmlChildElement.getName().equalsIgnoreCase(Constants.XML_TAG_AGENT_MODEL) )
		    {
			AgentModel model = AgentModel.parse( xmlChildElement );		
			agentModels.addModel( model );
		    }

		// load the policies that an agent will use unless

		// they are overridden at the group level

		else if ( xmlChildElement.getName().equals(Constants.XML_TAG_POLICIES) )
		    {
			Enumeration enumChildrenOfPolicies = xmlChildElement.getChildren().elements();					
			while( enumChildrenOfPolicies.hasMoreElements() ){
			    IXMLElement xmlChildOfPolicies = (IXMLElement)enumChildrenOfPolicies.nextElement();
			    if ( xmlChildOfPolicies.getName().equals(Constants.XML_TAG_POLICY_ASKING) )
				p_asking = (AskingPolicy)Class.forName( xmlChildOfPolicies.getContent() ).newInstance();
			    else if ( xmlChildOfPolicies.getName().equals(Constants.XML_TAG_POLICY_ANSWERING) )
				p_answering = (AnsweringPolicy)Class.forName( xmlChildOfPolicies.getContent() ).newInstance();
			    else if ( xmlChildOfPolicies.getName().equals(Constants.XML_TAG_POLICY_REFERRING) )
				p_referring = (ReferringPolicy)Class.forName( xmlChildOfPolicies.getContent() ).newInstance();
			    else if ( xmlChildOfPolicies.getName().equals(Constants.XML_TAG_POLICY_LEARNING) )
				p_learning = (LearningPolicy)Class.forName( xmlChildOfPolicies.getContent() ).newInstance();
			    else if ( xmlChildOfPolicies.getName().equals(Constants.XML_TAG_POLICY_CHANGING_NEIGHBORS) )
				p_changingNeighbors = (ChangingNeighborsPolicy)Class.forName( xmlChildOfPolicies.getContent() ).newInstance();
			    else if ( xmlChildOfPolicies.getName().equals(Constants.XML_TAG_POLICY_CACHING) )
				p_cachingPolicy = (CachingPolicy)Class.forName( xmlChildOfPolicies.getContent() ).newInstance();
			    else if ( xmlChildOfPolicies.getName().equals(Constants.XML_TAG_POLICY_QUERY_GENERATION) )
			    p_query = (QueryGenerationPolicy)Class.forName( xmlChildOfPolicies.getContent() ).newInstance();
			}
		    }				

	    }
		agent = new ServiceAgent( agentID);
		agent.setInterests( interests );
		agent.setExpertise( expertise );
		agent.setNeighbors( neighbors );

		agent.setAgentModels( agentModels );	

		agent.setAskingPolicy( p_asking );
		agent.setAnsweringPolicy( p_answering );
		agent.setReferringPolicy( p_referring );
		agent.setLearningPolicy( p_learning );
		agent.setChangingNeighborsPolicy( p_changingNeighbors );			
		agent.setCachingPolicy(p_cachingPolicy);
	    agent.setQueryGenerationPolicy(p_query);

	    }

	catch( Exception e ){
		e.printStackTrace();
	}			

	return agent;		
    }	

    
}// AgentParser
