package edu.ncsu.simulators.policies;

import edu.ncsu.simulators.Constants;
import edu.ncsu.simulators.util.Statistics;
import edu.ncsu.simulators.agent.AgentModel;
import edu.ncsu.simulators.agent.AgentModels;
import edu.ncsu.simulators.vectors.*;

import edu.ncsu.simulators.agent.Neighbors;

import java.util.Iterator;
import java.util.Random;
import java.util.Vector;

/**
 * Imitates a DNS Cache. Answers only if the item is in the cache. And if the item
 * is in the cache, then it is correct
 * 
 * 
 * @author <a href="mailto:pyolum@eos.ncsu.edu">Pinar Yolum</a>
 */
public class AnsweringPolicy_DNS_Cache implements AnsweringPolicy
{
    // the random number generator used by this agent
    private Random random = null;	
    private double correctPercentage = 1;	
    private double answeringThreshold = 0.5; 
    //=======================================================================
	
    public AnsweringPolicy_DNS_Cache(double giveAnswer, double returnCorrect)
    {
	random = Statistics.getRandomNumberGenerator();	
	answeringThreshold = giveAnswer; 
	correctPercentage = returnCorrect;
    }

    public AnsweringPolicy_DNS_Cache()
    {
	this(0.5, 1);
    }


    public Answer generateAnswer( AgentModel modelQuestioner, Query query, Expertise expertise, Interests interest, int myId, Neighbors neighbors)
    {
	Answer answer = null;
		
	double answerComponent = 0;

	boolean correct= false;

	Object temp = null;
		
	// Pick a random number, if it is below the answeringThreshold then answer. 
	// == Answer with answeringThreshold percentage

	if ( (random.nextDouble() <= answeringThreshold))
	    {
		// Pick a second number. If this is less than the correctPercentage 
		// give an answer of 1s otherwise give an answer of 0s.
		answer = new Answer();
			
		Iterator iteratorExpertiseComponents = expertise.iterator();

		correct = (random.nextDouble() <= correctPercentage);


		while( iteratorExpertiseComponents.hasNext() )
		    {
			temp = iteratorExpertiseComponents.next();
			if (correct) 
			    answerComponent = 1;
			else
			    answerComponent = 0;
			answer.add( new Double(answerComponent) );
		    }
	    }
	// otherwise, the agent will not answer the question
	else
	    {
		answer = null;
	    }

	return answer;	
    }
	
    //=======================================================================
}
