//==========================================================================
//	This source code is part of the Agent Simulation Test Bed system which
//	was developed for simulating interaction among agents who are attempting 
//	to acquire information from each other and learn about each other. 
//
//	The system was created at North Carolina State Univeristy, 
//	Department of Computer Science between September and December 2001. 
//	It was implemented by Michael J. Suralik based on the specifications 
//	developed by Pinar Yolum and Bin Yu, who were conducting research under 
//   the guidance of Dr. Munindar Singh. 
//==========================================================================

package edu.ncsu.simulators.policies;

import edu.ncsu.simulators.agent.AgentModel;
import edu.ncsu.simulators.agent.AgentModels;
import edu.ncsu.simulators.agent.Neighbors;
import edu.ncsu.simulators.vectors.Query;

import java.util.Iterator;

/**
 * Defines the interface between an agent and his policy for making decisions
 * about referring agents to each other.
 */
public interface ReferringPolicy
{	
	/**
	 * Returns an iterator over the IDs of the neighbors to whom the referring
	 * agent will refer the questioning agent. 
	 *
	 * All 4 parameters are copies of the actual objects that they represent.  
	 * They may be altered in order to facilitate generating the return value.  
	 * However, altering the parameters will have no effect outside this method.
	 *
	 * @param modelQuestioner the referring agent's model of the questioning agent.
	 * @param query the query that the questioning agent has made.
	 * @param neighbors all of the referring agent's neighbors.
	 * @param agentModels the referring agent's models of the other agents that it is aware of.
	 * @return an iterator over the IDs of neighbors to whom the referring agent will refer the questioning agent; 
	 * if, for any reason, the referring agent does not feel comfortable referring the questioning agent to any neighbors, then return NULL.
	 */ 
	public Iterator getNeighborsToReferTo( AgentModel modelQuestioner, Query query, Neighbors neighbors, AgentModels agentModels, int myId);	
}


