//==========================================================================
//	This source code is part of the Agent Simulation Test Bed system which
//	was developed for simulating interaction among agents who are attempting 
//	to acquire information from each other and learn about each other. 
//
//	The system was created at North Carolina State Univeristy, 
//	Department of Computer Science between September and December 2001. 
//	It was implemented by Michael J. Suralik based on the specifications 
//	developed by Pinar Yolum and Bin Yu, who were conducting research under 
//   the guidance of Dr. Munindar Singh. 
//==========================================================================

package edu.ncsu.simulators.util;

import edu.ncsu.simulators.agent.Agent;
import edu.ncsu.simulators.io.LogWriter;
import edu.ncsu.simulators.SimulationContext;

/**
 * The ErrorHandler prints error messages and gracefully shuts down the
 * simulation by releasing all resources before invoking System.exit().
 * All methods in this utility class should be public.
 */
public final class ErrorHandler
{	
	private static Agent[] agents = null;
	
	//========================================================================			

	private ErrorHandler()
	{	
	}
			
	//========================================================================	
	
	private static void exit()
	{	
		try
		{
			// close the Queue Connection
			SimulationContext.getQueueConnection().stop();			
			SimulationContext.getQueueConnection().close();
		}
		catch ( Exception e )
		{
			e.printStackTrace();
		}
		
		if ( agents != null )
		{
			// release all the resources that each agent holds
			for ( int id=0; id < agents.length; id++ )
			{
				agents[id].close();
			}					
		}
	
		// close any log file writers that are still open
		LogWriter.closeAllFiles();
		
		// end the program
		System.exit( 1 );		
	}
	
	//========================================================================		
	
	public static void exit( String errorMessage )
	{
		System.out.println( "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" );
		System.out.println( "          USER ERROR" );
		System.out.println( "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" );		
		System.out.println( "" );		
		System.out.println( errorMessage );		
		System.out.println( "" );		
		System.out.println( "Flushing and closing all FileWriters..." );
		System.out.println( "" );			

		exit();
	}
	
	//========================================================================		
	
	public static void exit( Exception e, String errorMessage )
	{
		System.out.println( "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" );
		System.out.println( "          EXCEPTION" );
		System.out.println( "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" );		
		System.out.println( "" );		
		System.out.println( errorMessage );		
		System.out.println( "" );		
		e.printStackTrace();	
		System.out.println( "" );
		System.out.println( "Flushing and closing all FileWriters..." );
		System.out.println( "" );		
		
		exit();
	}
	
	//========================================================================		
	
	public static void exit( Exception e )
	{
		System.out.println( "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" );
		System.out.println( "          EXCEPTION" );
		System.out.println( "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" );		
		System.out.println( "" );		 
		e.printStackTrace();
		System.out.println( "" );		
		System.out.println( "Flushing and closing all FileWriters..." );
		System.out.println( "" );		
	
		exit();
	}	
	
	//========================================================================		
	
	public static void setAgents( Agent[] _agents )
	{
		agents = _agents;
	}
	
	//========================================================================	
}