//==========================================================================
//	This source code is part of the Agent Simulation Test Bed system which
//	was developed for simulating interaction among agents who are attempting 
//	to acquire information from each other and learn about each other. 
//
//	The system was created at North Carolina State Univeristy, 
//	Department of Computer Science between September and December 2001. 
//	It was implemented by Michael J. Suralik based on the specifications 
//	developed by Pinar Yolum and Bin Yu, who were conducting research under 
//   the guidance of Dr. Munindar Singh. 
//==========================================================================

package edu.ncsu.simulators.util;

import java.lang.Integer;
import java.util.Collection;
import java.util.Iterator;
import java.util.Random;
import java.util.Vector;

/**
 * The RandomIterator class provides methods for generating iterators that will 
 * iterate over a collection of existing objects in a random order.  All methods 
 * in this utility class should be public.
 */
public final class RandomIterator
{
	private static Random randomNumberGenerator = null;
	
	//========================================================================			
	
	private RandomIterator()
	{	
	}	
	
	//=======================================================================		
	
	public static Iterator getRandomIterator( Object[] objects )
	{	
		Vector vector = new Vector( objects.length );
		
		for ( int i=0; i < objects.length; i++ )
		{
			vector.addElement( objects[i] );
		}
		
		return getRandomIterator( vector );
	}
	
	//=======================================================================		
	
	public static Iterator getRandomIterator( Collection collection )
	{	
		int size = collection.size();
		
		Vector randomlyOrderedVector = new Vector( size );		
		Vector positionsAvailable = new Vector( size );			

		Random randomNumberGenerator = Statistics.getRandomNumberGenerator();		
		
		// initialize the randomly ordered vector with dummy values so that
		// we can use the replace methods instead of the insert methods
		// to fill the vector.
		for ( int i=0; i<size; i++ )
		{
			randomlyOrderedVector.addElement( new Integer(-1) );
		}
		
		// initialize a vector containing all the available positions
		// in the randomly ordered vector.	
		for ( int i=0; i<size; i++ )
		{
			positionsAvailable.add( i, new Integer(i) );
		}
		
		// Create an iterator to return all of the objects in the collection
		// one at a time.
		Iterator objects = collection.iterator();
		
		// Iterate over all of the collection objects, 
		// putting each of them into a random position in the randomlyOrderedVector.
		while ( positionsAvailable.size() > 0 )
		{		
			// randomly select the index of one of the positions that has
			// not been taken yet
			double randomNumber = randomNumberGenerator.nextDouble();	
			int index = (int)(randomNumber * positionsAvailable.size());
			
			// remove the position number so that it can not be used again
			int position = ((Integer)positionsAvailable.remove( index )).intValue();
		
			// insert the next object from the collection into the randomly
			// ordered vector at the randomly selected position.
			randomlyOrderedVector.setElementAt( objects.next(), position );	
		}
	
		return randomlyOrderedVector.iterator();
	}
		
	
	//========================================================================
}